import configparser
import os
import re
import sys
import ast
import time
import json
from subprocess import Popen
from . import viihdeapi
from . import naming_rules
os.system('color')

def main():
    config_path = os.path.join(os.environ['APPDATA'], 'viihdecli', 'settings.ini')
    config = configparser.ConfigParser()
    config.read(config_path)
    dl_settings = config['Download settings']
    dl_folder = dl_settings['download folder']
    folder_structure = dl_settings.getboolean('folder structure')

    arguments = sys.argv[1:]
    dl_list = []
    for x in range(len(arguments)):
        if arguments[x].isnumeric():
            dl_list.append(arguments[x])
        else:
            header_string = ' '.join(arguments[x:])
            break

    queue_size = len(dl_list) - 1
    headers = ast.literal_eval(header_string)

    if dl_folder == '':
        print('Latauskansio: ' + os.getcwd() + '\\')
    elif not dl_folder.endswith('\\'):
        dl_folder += '\\'
        if not os.path.isdir(dl_folder): os.makedirs(dl_folder)
        print('Latauskansio: ' + dl_folder)
    else:
        if not os.path.isdir(dl_folder): os.makedirs(dl_folder)
        print('Latauskansio: ' + dl_folder)

    if folder_structure:
        with open(os.path.join(os.environ['APPDATA'], 'viihdecli', 'kansiot.json')) as f:
            folders = json.load(f)

    for recording in dl_list:
        filename = dl_folder
        recording_info = viihdeapi.get_recording_info(recording, headers, dl_settings['platform'])
        recording_url = viihdeapi.get_recording_url(recording, headers, dl_settings['platform'])
        if folder_structure and 'folderId' in recording_info:
            if len(folders[str(recording_info['folderId'])][1]) > 1:
                for x in folders[str(recording_info['folderId'])][1][1:]:
                    filename = os.path.join(filename, re.sub(r'[\\/*?:"<>|]', "_", folders[str(x)][2]))
            filename = os.path.join(filename, re.sub(r'[\\/*?:"<>|]', "_", folders[str(recording_info['folderId'])][2]))
            if not os.path.isdir(filename): os.makedirs(filename)
        filename = os.path.join(filename, naming_rules.create_filename(recording_info))
        if dl_settings.getboolean('prompt overwrite') and os.path.isfile(filename + '.' + dl_settings['file extension']):
            overwrite = input('\nTiedosto \033[91m' + filename + '.' + dl_settings['file extension']
                              + '\033[39m on jo olemassa. Ladataanko se silti uudelleen (k/e)? ').lower()
            if not overwrite in ['k', 'y']:
                print('Tallennetta ei ladata uudestaan.')
                queue_size -= 1
                continue
        cmd = 'viihdexdl -c \"' + config_path + '\" -y '
        if dl_settings.getboolean('external subtitles'):
            cmd += '-e '
        cmd += '\"' + recording_url + '\" \"' + filename + '\"'

    # Save description
        if dl_settings.getboolean('save description'):
            file = open(filename + '.txt', 'w')
            if 'description' in recording_info:
                file.write(recording_info['description'])
            else:
                file.write('Tallenteella ei ole kuvausta.')
            file.close()

        if queue_size == 1:
            print('\nJonossa \033[36m' + str(queue_size) + '\033[39m tallenne.')
        elif queue_size > 1:
            print('\nJonossa \033[36m' + str(queue_size) + '\033[39m tallennetta.')
        print()
        Popen(cmd).wait()
        print()
        time.sleep(2)
        queue_size -= 1
    print()
    a = input('Tallenteet ladattu.')



import configparser
import getpass
import os
import webbrowser
import keyring
os.system('color')

def create_config(config_path, config = None, write_config = True):
    config_changed = False
    default_config = configparser.ConfigParser()
    default_config['Login information'] = { 'service name': 'Elisa Viihde API',
                                            'username': '',
                                            'auto login': 'false' }
    default_config['General settings'] = { 'print help at start': 'true',
                                           'move prompt min': '1',
                                           'print at start': 'true',
                                           'print all': 'true',
                                           'sorting key': 'startTime',
                                           'sorting dir': 'asc',
                                           'recycle duplicates': 'false' }
    default_config['Download settings'] = { 'player': '\"C:\\Program Files\\MPC-HC\\mpc-hc64.exe\"',
                                            'platform': 'ios',
                                            'download folder': '',
                                            'folder structure': 'false',
                                            'season episode filename': 'false',
                                            'audio languages': 'fin, dut',
                                            'subtitle languages': 'fi, nl',
                                            'default audio': 'fin, dut',
                                            'default subtitle': 'fin, dut',
                                            'visual impaired': 'dut',
                                            'hearing impaired': 'dut',
                                            'maximum bandwidth': '0',
                                            'maximum width': '0',
                                            'maximum height': '0',
                                            'file extension': 'mkv',
                                            'external subtitles': 'true',
                                            'ffmpeg options': '-v error -stats',
                                            'ffmpeg video codec': 'copy',
                                            'ffmpeg audio codec': 'copy',
                                            'save description': 'true',
                                            'prompt overwrite': 'true' }
    defaults = [{'home': '0'}, {'esimerkki': 'C:\\Lataukset'}, {'nelonen': 'c nelonen | hero | liv | jim'}]
    if not config:
        config = default_config
        config_changed = True
    else:
        for s in ['Login information', 'General settings', 'Download settings']:
            if s in config:
                for k in default_config[s]:
                    if k in config[s]:
                        default_config[s][k] = config[s][k]
                    else:
                        config_changed = True
            else:
                config_changed = True
    for i, s in enumerate(['Folder shortcuts', 'Download folders', 'Filter shortcuts']):
        if s in config and not len(config[s]) == 0:
            default_config[s] = config[s]
        else:
            default_config[s] = defaults[i]
            config_changed = True
    if write_config and config_changed:
        if not os.path.exists(config_path):
            os.mkdir(config_path)
        with open(os.path.join(config_path, 'settings.ini'), 'w') as configfile:
            default_config.write(configfile)
    return default_config

def main():
    from . import version
    version = version.__version__
    print('ViihdeCLI ' + version + ' (c) 2021-2024 Qotscha\n')
    # Load config file.
    config_folder = os.path.join(os.environ['APPDATA'], 'viihdecli')
    config_path = os.path.join(config_folder, 'settings.ini')
    if not os.path.exists(config_path):
        config = create_config(config_folder)
    else:
        config = configparser.ConfigParser()
        config.read(config_path)
        config = create_config(config_folder, config)
    # config_path = os.path.join(config_folder, 'settings.ini')
    config_changed = False
    columns_path = os.path.join(config_folder, 'columns.ini')
    if not os.path.isfile(columns_path):
        columns = configparser.ConfigParser()
        columns['Recordings'] = { 'spacing': '3',
                                  'negative': 'true',
                                  'start day': 'true',
                                  'start date': 'true',
                                  'start time': 'true',
                                  'end day': 'false',
                                  'end date': 'false',
                                  'end time': 'true',
                                  'channel': '9',
                                  'duration': 'true',
                                  'name': '41',
                                  'folder': '12',
                                  'show imdb': 'true',
                                  'show live': 'true' }
        columns['Recycle bin'] = { 'spacing': '3',
                                   'negative': 'false',
                                   'removal day': 'false',
                                   'removal date': 'true',
                                   'removal time': 'false',
                                   'start day': 'true',
                                   'start date': 'true',
                                   'start time': 'true',
                                   'end day': 'false',
                                   'end date': 'false',
                                   'end time': 'false',
                                   'channel': '9',
                                   'duration': 'true',
                                   'name': '43',
                                   'folder': '13',
                                   'show imdb': 'true',
                                   'show live': 'true' }
        with open(columns_path, 'w') as configfile:
            columns.write(configfile)

    service_name = config['Login information']['service name']

    client_secret = 'nZhkFGz8Zd8w'
    api_key = 'Fjv8TK75OStLIz7Sc6jw6PvYQiIEgDtk'
    platform = config['Download settings']['platform']

    # Read/write username.
    save_username = ''
    username = config['Login information']['username']
    if username == '':
        username = input('Käyttäjätunnus: ')
        save_username = input('Tallennetaanko käyttäjätunnus (k/e)? ').lower()
        if save_username in ['k', 'y']:
            config['Login information']['username'] = username
            config_changed = True
    else:
        if config['Login information'].getboolean('auto login'):
            print('Kirjaudutaan käyttäjänä ' + username + '.')
        else:
            new_username = input('Kirjaudu käyttäjänä ' + username + ' painamalla ENTER '
                                'tai anna uusi käyttäjätunnus: ')
            if not new_username == '':
                config['Login information']['username'] = ''
                if not keyring.get_password(service_name, username) is None:
                    keyring.delete_password(service_name, username)
                username = new_username
                save_username = input('Tallennetaanko käyttäjätunnus (k/e)? ').lower()
                if save_username in ['k', 'y']:
                    config['Login information']['username'] = username
                config_changed = True

    # Load/save password.
    save_password = ''
    password = keyring.get_password(service_name, username)
    if password is None:
        print()
        password = getpass.getpass(prompt='Salasana: ')
        if config['Login information'].getboolean('auto login') or save_username in ['k', 'y']:
            save_password = input('Tallennetaanko salasana (k/e)? ').lower()

    from . import viihdeapi

    try:
        headers = viihdeapi.login(client_secret, api_key, username, password)
        print('\nKirjautuminen onnistui.')
        if config_changed:
            with open(config_path, 'w') as configfile:
                config.write(configfile)
        if save_password in ['k', 'y']:
            keyring.set_password(service_name, username, password)
    except:
        print('\nKirjautuminen ei onnistunut. Tarkista käyttäjätunnus ja salasana.')
        config['Login information']['username'] = ''
        with open(config_path, 'w') as configfile:
            config.write(configfile)
        if not keyring.get_password(service_name, username) is None:
            keyring.delete_password(service_name, username)
        return

    from . import viihde

    while True:

        print('\nLatauskansio: \033[32m' + config['Download settings']['download folder'] + '\033[39m')
        # mode = input('\nValitse toimintatila [p] Liitä, [f] Kansiot, [a] Kaikki tallenteet tai [e] Ohjelmaopas: ').lower()
        mode = input('\nValitse toimintatila:\n   [d] Poista kirjautumistiedot\n   [p] Liitä\n   [f] Kansiot\n   [a] Kaikki tallenteet\
                      \n   [r] Roskakori\n   [c] Avaa asetustiedosto Muistiossa\n   [q] Sulje ohjelma\nValinta: ').lower()
        if mode == 'p':
            print()
            viihde.handle_input(headers)
        elif mode == 'i':
            account_info = viihdeapi.get_account_info(headers)
            print(account_info)
            account_quota = viihdeapi.get_quota(headers, platform)
            print(account_quota)
        elif mode == 'f':
            folder_tree = viihdeapi.get_folder_tree(headers, platform)
            viihde.handle_folders(folder_tree, headers)
        elif mode == 'a':
            folder_tree = viihdeapi.get_folder_tree(headers, platform)
            recordings = viihdeapi.get_all_recordings(config['General settings']['sorting key'], config['General settings']['sorting dir'], headers, platform)
            viihde.handle_recordings(folder_tree, recordings['recordings'], headers, config['General settings'].getboolean('print at start'))
        elif mode == 'r':
            folder_tree = viihdeapi.get_folder_tree(headers, platform)
            recordings = viihdeapi.get_recycle(config['General settings']['sorting key'], config['General settings']['sorting dir'], headers, platform)
            viihde.handle_recordings(folder_tree, recordings['recordings'], headers, config['General settings'].getboolean('print at start'), trash = True)
        elif mode == 'c':
            print('\nAvataan ' + config_path + ' Muistiossa.')
            webbrowser.open(config_path)
        elif mode == 'd':
            if not keyring.get_password(service_name, username) is None:
                keyring.delete_password(service_name, username)
                print('\nPoistettiin tallennettu salasana.')
            config['Login information']['username'] = ''
            with open(config_path, 'w') as configfile:
                config.write(configfile)
            print('Poistettiin käyttäjätunnus asetustiedostosta')
        else:
            break

if __name__ == "__main__":
    main()



import locale
import os
import textwrap
# import time
from datetime import datetime, timedelta
os.system('color')
locale.setlocale(locale.LC_ALL, '')

def get_date_format(spacing, wd, sd, st, long, short):
    if sd:
        if st:
            f = 16
            i = (0, 16)
        else:
            f = 10
            i = (0, 10)
    else:
        if st:
            f = 5
            header = short
            i = (11, 16)
        else:
            f = 0
            header = ''
            i = None
    if wd:
        if f > 0:
            f += 1
        f += max([len(datetime.fromisoformat(f'2000-01-0{x}').strftime('%a')) for x in range(1,8)])
    if f:
        f += spacing
    if f > len(long):
        header = long
    elif f > len(short):
        header = short
    elif f:
        header = short[0] + '.'
    return f, header, wd, i

def get_format_list(n, ms, columns, trash):
    c = [[len(str(n)) + ms, '#'], [len(str(n)) + 1 + ms, '-#'] if columns.getboolean('negative') else [0, ''],
         get_date_format(ms, columns.getboolean('start day'), columns.getboolean('start date'),
                         columns.getboolean('start time'), 'Alkoi', 'Alk.'),
         get_date_format(ms, columns.getboolean('end day'), columns.getboolean('end date'),
                         columns.getboolean('end time'), 'Päättyi', 'Lopp.'),
         [int(columns['channel']) + ms, 'Kanava'] if int(columns['channel']) else [0, ''],
         [4 + ms, 'Kesto'] if columns.getboolean('duration') else [0, ''],
         [int(columns['name']) + ms if int(columns['name']) > 20 else 20 + ms, 'Nimi'],
         [int(columns['folder']), 'Kansio'] if int(columns['folder']) else [0, '']]
    # if not c[2][0][0]: c[2][1] = ''
    # if not c[3][0][0]: c[3][1] = ''
    if c[4][0] < 7: c[4][1] = 'Kan.'
    if c[5][0] < 6: c[5][1] = 'K.'
    if columns.getboolean('show imdb'): c[6][1] += ' [IMDB-arvosana]'
    if trash:
        c.append(get_date_format(ms, columns.getboolean('removal day'), columns.getboolean('removal date'),
                  columns.getboolean('removal time'), 'Poistuu', 'Poist.'))
        # if not c[8][0][0]: c[8][1] = ''
    return c

def show_terminal_width(config, n, trash):
    if trash:
        columns = config['Recycle bin']
    else:
        columns = config['Recordings']
    ms = int(columns['spacing'])
    c = get_format_list(n, ms, columns, trash)
    t_width = os.get_terminal_size()[0]
    n_width = sum([i[0] for i in c])
    print(f'Tämänhetkinen terminaalin leveys: {t_width}')
    print(f'Tarvittava terminaalin leveys:    {n_width}')

def print_recordings(config, folder_dict, recording_list, hl_set = set(), print_descriptions = False, trash = False):
    # stime = time.time()
    n = len(recording_list)
    if trash:
        columns = config['Recycle bin']
    else:
        columns = config['Recordings']
    ms = int(columns['spacing'])
    c = get_format_list(n, ms, columns, trash)
    t_width, t_lines = os.get_terminal_size()
    n_width = sum([i[0] for i in c])
    c[7][0] = max(t_width - n_width + c[7][0], 0)
    if not c[7][0]: c[7][1] = ''
    c = tuple(tuple(i) for i in c)
    # for col in c:
        # print(col)
    if trash:
        header_row = f'{c[0][1]:<{c[0][0]}}{c[1][1]:<{c[1][0]}}{c[8][1]:<{c[8][0]}}{c[2][1]:<{c[2][0]}}{c[3][1]:<{c[3][0]}}' \
                     f'{c[4][1]:<{c[4][0]}}{c[5][1]:<{c[5][0]}}{c[6][1]:<{c[6][0]}}{c[7][1]:<{c[7][0]}}'
    else:
        header_row = f'{c[0][1]:<{c[0][0]}}{c[1][1]:<{c[1][0]}}{c[2][1]:<{c[2][0]}}{c[3][1]:<{c[3][0]}}{c[4][1]:<{c[4][0]}}' \
                     f'{c[5][1]:<{c[5][0]}}{c[6][1]:<{c[6][0]}}{c[7][1]:<{c[7][0]}}'
    y = 0
    z = - n if c[1][1] else ''
    print('\n\033[92m' + header_row + '\033[39m')
    total_duration = 0

    for x in recording_list:
        start_day = f"{datetime.fromisoformat(x['startTime']).strftime('%a')} " if c[2][2] else ''
        start_date = f"{start_day}{x['startTime'][c[2][3][0]:c[2][3][1]]}" if c[2][3] else start_day
        end_day = f"{datetime.fromisoformat(x['endTime']).strftime('%a')} " if c[3][2] else ''
        end_date = f"{end_day}{x['endTime'][c[3][3][0]:c[3][3][1]]}" if c[3][3] else end_day
        if trash:
            removal_day = f"{datetime.fromisoformat(x['removalDate']).strftime('%a')} " if c[8][2] else ''
            removal_date = f"\033[31m{removal_day}{x['removalDate'][c[8][3][0]:c[8][3][1]]}" if c[8][3] else f"\033[31m{removal_day}"
        dur = str(timedelta(seconds=x['duration'])).rsplit(':',1)[0] if c[5][0] else ''
        channel = x['channelName'][:c[4][0] - ms] if c[4][0] else ''
        extra = 0
        padding = c[6][0] + 5
        name_length = c[6][0] - ms
        live = ''
        imdb = ''
        hl = ''
        if x['programId'] in hl_set:
            hl = '\033[31m'
            padding += 5
        if x['recordingState'] == 'ongoing' and columns.getboolean('show live'):
            live = ' \033[91m[LIVE]'
            extra += 7
            padding += 5
        if 'imdbRating' in x and columns.getboolean('show imdb'):
            imdb = ' \033[33m[' + str(x['imdbRating']) + ']'
            extra += 6
            padding += 5
        name_length -= extra
        recording_name = f"{hl}{x['name'][:name_length].rstrip()}{imdb}{live}"
        # recording_name = f"{hl}{x['name'][:name_length-3].rstrip()}...{imdb}{live}" if name_length < len(x['name']) else f"{hl}{x['name'].rstrip()}{imdb}{live}"
        # fl = len(folder_dict[x['folderId']][0])
        if c[7][0]:
            folder_name = folder_dict[x['folderId']][0]
            if c[7][0] < len(folder_name):
                if folder_dict[x['folderId']][1] and len(folder_dict[x['folderId']][1]) > 1:
                    folder_start = '.. /'
                    for i in range(1, len(folder_dict[x['folderId']][1])):
                        folder_name = f"{folder_start}{folder_name.split('/', i)[i]}"
                        if len(folder_name) <= c[7][0]:
                            break
                        folder_start += ' .. /'
                folder_name = folder_name[:c[7][0]]
        else:
            folder_name = ''
        # folder_name = folder_dict[x['folderId']][0][:c[7][0]] if c[7][0] else ''
        if print_descriptions:
            recording_name += '\033[39m'
            if trash:
                removal_date += '\033[39m'
                info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{removal_date:<{c[8][0]+10}}{start_date:<{c[2][0]}}{end_date:<{c[3][0]}}" \
                           f"{channel:<{c[4][0]}}{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
            else:
                info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{start_date:<{c[2][0]}}{end_date:<{c[3][0]}}{channel:<{c[4][0]}}" \
                           f"{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
            print(info_row)
            if 'description' in x:
                desc_lines = textwrap.wrap(x['description'], width = t_width - 15)
                for line in desc_lines:
                    print(f"\033[36m{'':<{c[0][0] + c[1][0]}}{line}\033[39m")
            else:
                print(f"\033[36m{'':<{c[0][0] + c[1][0]}}{'Ei kuvausta'}\033[39m")
        else:
            if (y % 2) == 1:
                recording_name += '\033[36m'
                if trash:
                    removal_date += '\033[36m'
                    info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{removal_date:<{c[8][0]+10}}{start_date:<{c[2][0]}}" \
                               f"{end_date:<{c[3][0]}}{channel:<{c[4][0]}}{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
                else:
                    info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{start_date:<{c[2][0]}}{end_date:<{c[3][0]}}{channel:<{c[4][0]}}" \
                               f"{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
                print('\033[36m' + info_row + '\033[39m')
            else:
                recording_name += '\033[39m'
                if trash:
                    removal_date += '\033[39m'
                    info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{removal_date:<{c[8][0]+10}}{start_date:<{c[2][0]}}" \
                               f"{end_date:<{c[3][0]}}{channel:<{c[4][0]}}{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
                else:
                    info_row = f"{y:<{c[0][0]}}{z:<{c[1][0]}}{start_date:<{c[2][0]}}{end_date:<{c[3][0]}}{channel:<{c[4][0]}}" \
                               f"{dur:<{c[5][0]}}{recording_name:<{padding}}{folder_name:<{c[7][0]}}"
                print(info_row)
        y += 1
        if c[1][1]: z += 1
        total_duration += x['duration']
    if n > t_lines - 4:
        print('\033[92m' + header_row + '\033[39m')
    hours_seconds = divmod(total_duration, 3600)
    minutes = hours_seconds[1] // 60
    print('Suodatettujen tallenteiden kesto yhteensä ' + str(hours_seconds[0]) + ' tuntia, ' + str(minutes) + ' minuuttia.')
    # print(f'Aikaa kului {time.time() - stime} sekuntia.')
    # print(f'Terminaalin leveys: {t_width}')
    # print(f'Tarvittava terminaalin leveys: {n_width}')
    print()



import configparser
import os
import re
import json
import webbrowser
import time
from datetime import date, timedelta
from subprocess import Popen, CREATE_NEW_CONSOLE
from . import viihdeapi
from . import command_strings
from . import viihdehelp
from .filter_recordings import filter_recordings
from .print_recordings import print_recordings, show_terminal_width
from .duplicates import list_duplicates
os.system('color')

config_path = os.path.join(os.environ['APPDATA'], 'viihdecli', 'settings.ini')
config = configparser.ConfigParser()
config.read(config_path)
platform = config['Download settings']['platform']

columns_path = os.path.join(os.environ['APPDATA'], 'viihdecli', 'columns.ini')
columns = configparser.ConfigParser()
columns.read(columns_path)

def create_number_list(number_string, last_item):
    number_string = number_string.replace(',', ' ')
    raw_number_list = number_string.split()
    number_list = []
    for x in raw_number_list:
        to_add = []
        reverse_list = False
        if x.startswith('!'):
            reverse_list = True
            x = x.strip('!')
        y = x.split(':')
        if len(y) == 1:
            following = 0
            previous = 0
            while x[-1] in ['+', '-']:
                if x[-1] == '+':
                    following += 1
                else:
                    previous += 1
                x = x[:-1]
            x_ = int(x)
            x_ = x_ + last_item + 1 if x_ < 0 else x_
            to_add = list(range(x_ - previous, x_ + following + 1))
        elif len(y) == 2:
            y[0] = 0 if not y[0] else int(y[0])
            y[1] = last_item if not y[1] else int(y[1])
            y = [z + last_item + 1 if z < 0 else z for z in y]
            to_add = list(range(y[0], y[1] + 1)) if y[0] <= y[1] else list(range(y[0], y[1] - 1, -1))
        elif len(y) == 3:
            step = 1 if not y[1] else int(y[1])
            y.pop(1)
            if step < 0:
                y[0] = last_item if not y[0] else int(y[0])
                y[1] = 0 if not y[1] else int(y[1])
                y = [z + last_item + 1 if z < 0 else z for z in y]
                to_add = list(range(y[0], y[1] - 1, step))
            else:
                y[0] = 0 if not y[0] else int(y[0])
                y[1] = last_item if not y[1] else int(y[1])
                y = [z + last_item + 1 if z < 0 else z for z in y]
                to_add = list(range(y[0], y[1] + 1, step)) if y[1] > y[0] else list(range(y[0], y[1] - 1, -step))
        if reverse_list:
            to_add.reverse()
        if to_add: number_list += to_add
    return number_list

def set_filter_shortcut(shortcut, f_list, so):
    for i, x in enumerate(f_list):
        if len(f_list) == 1:
            config['Filter shortcuts'][shortcut] = x
        else:
            config['Filter shortcuts'][f'{shortcut}_{i}'] = x
    if so:
        config['Filter shortcuts'][f'{shortcut}_so_'] = so
    with open(config_path, 'w') as configfile:
        config.write(configfile)
    print('Luotiin pikavalinta \033[92m' + shortcut + '\033[39m suotimille')
    for x in f_list:
        print('   ' + x)
    print(f'   Sääntö: {so}')

def list_filter_shortcuts():
    filter_dict = dict(config['Filter shortcuts'])
    f_items = list(filter_dict.items())
    print('\n\033[92m' + '{:<20}{}'.format('Pikavalinta', 'Komento') + '\033[39m')
    f_group = False
    f_i = 0
    for i, x in enumerate(f_items):
        if x[0].endswith('_0') and f'{x[0][:-1]}1' in filter_dict:
            print(f'\033[32m{x[0][:-2]}\033[39m')
            print('   {:<20}{}'.format(x[0], x[1]))
            f_i = 1
        elif f_i > 0 and x[0].endswith(f'_{f_i}'):
            print('   {:<20}{}'.format(x[0], x[1]))
            f_i += 1
        elif f_i > 0 and x[0].endswith('_so_'):
            print(f'   Sääntö: {x[1]}')
        else:
            f_i = 0
            print('{:<20}{}'.format(x[0].strip('_0'), x[1]))
    print()

def set_folder_shortcut(folder_id, shortcut, folder_name = '', print_msg = True):
    config['Folder shortcuts'][shortcut] = str(folder_id)
    with open(config_path, 'w') as configfile:
        config.write(configfile)
    if print_msg:
        print('Luotiin pikavalinta \033[92m' + shortcut + '\033[39m kansiolle \033[32m' + folder_name + '\033[39m.')

def list_folder_shortcuts(folder_dict):
    shortcut_dict = dict(config['Folder shortcuts'])
    print('\n\033[92m' + '{:<20}{}'.format('Pikavalinta', 'Kansio') + '\033[39m')
    for x in shortcut_dict:
        if not folder_dict.get(int(shortcut_dict[x])):
            config.remove_option('Folder shortcuts', x)
            with open(config_path, 'w') as configfile:
                config.write(configfile)
        else:
            print('{:<20}{}'.format(x, folder_dict.get(int(shortcut_dict[x]))[0]))
    print()

def set_dl_folder(dl_folder):
    config['Download settings']['download folder'] = dl_folder
    print('Latauskansio: \033[32m' + dl_folder + '\033[39m')
    with open(config_path, 'w') as configfile:
        config.write(configfile)

def set_dl_folder_shortcut(dl_folder, shortcut):
    config['Download folders'][shortcut] = dl_folder
    with open(config_path, 'w') as configfile:
        config.write(configfile)
    print('Luotiin pikavalinta \033[92m' + shortcut + '\033[39m latauskansiolle \033[32m' + dl_folder + '\033[39m.')

def list_dl_folders():
    print('\nNykyinen latauskansio: \033[32m' + config['Download settings']['download folder'] + '\033[39m')
    dl_folder_dict = dict(config['Download folders'])
    print('\n\033[92m' + '{:<20}{}'.format('Pikavalinta', 'Latauskansio') + '\033[39m')
    for x in dl_folder_dict:
        print('{:<20}{}'.format(x, dl_folder_dict[x]))
    print()

def play_recording(recording_id, headers, get_url_only = False):
    recording_url = viihdeapi.get_recording_url(recording_id, headers, platform)
    if get_url_only:
        return recording_url
    # cmd = 'wscript.exe \"C:\\Ohjelmat\\Elisa Viihde Media Manager\\run_mpc-hc_hls.vbs\" ' + recording_url
    cmd = config['Download settings']['player'] + ' ' + recording_url
    # print(cmd)
    Popen(cmd)

def download_folder(folder_id, headers):
    folder_info = viihdeapi.get_recordings(folder_id, 'startTime', 'asc', headers, platform)
    cmd = 'viihdedl '
    for x in folder_info['recordings']:
        cmd += str(x['programId']) + ' '
    # print(cmd)
    cmd += str(headers)
    Popen(cmd, creationflags=CREATE_NEW_CONSOLE)

def handle_folders(folder_tree, headers, move_recordings = False, folder_dict = None):
    global platform
    if not folder_dict:
        folder_dict = create_folder_dict(folder_tree)
    folder_numbers = []
    current_folder = list_folders(folder_dict, folder_tree, folder_numbers)
    if config['General settings'].getboolean('print help at start'):
        viihdehelp.print_help('folders')
        print()
    while True:
        try:
            folder_number = input('Mene kansioon X tai anna muu komento: ').strip()
            if folder_number == '':
                break
            if folder_number == command_strings.SHOW_HELP:
                print()
                viihdehelp.print_help('folders')
                print()
                continue
            elif folder_number == command_strings.SHOW_QUOTA:
                account_quota = viihdeapi.get_quota(headers, platform)
                seconds = account_quota['secondsLeft']
                hours_seconds = divmod(seconds, 3600)
                minutes = hours_seconds[1] // 60
                print(f'Tallennustilaa jäljellä {hours_seconds[0]} tuntia, {minutes} minuuttia')
                continue
            elif folder_number == command_strings.ROOT_FOLDER:
                folder_numbers = []
            elif folder_number == command_strings.FOLDER_UP:
                folder_numbers.pop()
            elif folder_number == command_strings.LIST_FOLDERS:
                pass
            elif folder_number == command_strings.REFRESH_RECORDINGS:
                folder_tree = viihdeapi.get_folder_tree(headers, platform)
                folder_dict = create_folder_dict(folder_tree)
            elif folder_number.isnumeric():
                folder_numbers.append(int(folder_number))
            elif folder_number.startswith(command_strings.NEW_FOLDER):
                new_folder_name = folder_number.split(' ', 1)[1]
                new_folder = viihdeapi.create_folder(new_folder_name, str(current_folder['id']), headers, platform)
                folder_tree = viihdeapi.get_folder_tree(headers, platform)
                folder_dict = create_folder_dict(folder_tree)
            elif folder_number == command_strings.LIST_RECORDINGS:
                recordings = viihdeapi.get_recordings(str(current_folder['id']), 'startTime', 'asc', headers, platform)
                if move_recordings:
                    refresh_folders = handle_recordings(folder_tree, recordings['recordings'], headers, True, True, current_folder['id'])
                else:
                    refresh_folders = handle_recordings(folder_tree, recordings['recordings'], headers, True, False, current_folder['id'])
                if refresh_folders:
                    folder_tree = viihdeapi.get_folder_tree(headers, platform)
            elif folder_number == command_strings.DOWNLOAD_FOLDER:
                download_folder(str(current_folder['id']), headers)
                continue
            elif folder_number.startswith(command_strings.SET_SHORTCUT):
                set_folder_shortcut(current_folder['id'], folder_number.split(' ', 1)[1], folder_dict[current_folder['id']][0])
                continue
            elif folder_number == command_strings.LIST_FOLDER_SHORTCUTS:
                list_folder_shortcuts(folder_dict)
                continue
            elif folder_number.startswith(command_strings.DL_FOLDER_SHORTCUT):
                set_dl_folder(config['Download folders'][folder_number.split(' ', 1)[1]])
                continue
            elif folder_number.startswith(command_strings.SET_DL_FOLDER):
                set_dl_folder(folder_number.split(' ', 1)[1])
                continue
            elif folder_number.startswith(command_strings.SET_DL_FOLDER_SHORTCUT):
                f_split = folder_number.split(' ', 1)[1].split('|', 1)
                if len(f_split) == 1:
                    f_split.append(config['Download settings']['download folder'])
                set_dl_folder_shortcut(f_split[1].strip(), f_split[0].strip())
                continue
            elif folder_number == command_strings.LIST_DL_FOLDERS:
                list_dl_folders()
                continue
            elif folder_number.startswith(command_strings.DELETE_FOLDER):
                if folder_number == command_strings.DELETE_FOLDER:
                    del_folder = current_folder
                    folder_numbers.pop()
                else:
                    del_folder = current_folder['folders'][int(folder_number.split()[1])]
                del_confirmation = input('Haluatko varmasti poistaa kansion ' + str(del_folder['name']) + ' tallenteineen (k/e)? ')
                if del_confirmation in ['k', 'y']:
                    if viihdeapi.delete_folder(str(del_folder['id']), headers, platform) == 200:
                        print('Kansio ' + str(del_folder['name']) + ' poistettiin.')
                        folder_tree = viihdeapi.get_folder_tree(headers, platform)
                        folder_dict.pop(del_folder['id'])
            elif folder_number == command_strings.RELOAD_CONFIG:
                config['Folder shortcuts'] = {}
                config['Download folders'] = {}
                config.read(config_path)
                columns.read(columns_path)
                platform = config['Download settings']['platform']
                print('Asetukset päivitetty.')
                continue
            elif folder_number == command_strings.OPEN_CONFIG:
                print('Avataan ' + config_path + ' Muistiossa.')
                print('Muokattuasi asetustiedostoa kirjoita ' + command_strings.RELOAD_CONFIG + ' päivittääksesi asetukset.')
                webbrowser.open(config_path)
            elif folder_number == command_strings.OPEN_COLUMNS:
                print('Avataan ' + columns_path + ' Muistiossa.')
                print('Muokattuasi asetustiedostoa kirjoita ' + command_strings.RELOAD_CONFIG + ' päivittääksesi asetukset.')
                webbrowser.open(columns_path)
            else:
                print('Komentoa ei tunnistettu.')
                continue
            current_folder = list_folders(folder_dict, folder_tree, folder_numbers)
        except:
            print('Virheellinen komento.')
            # raise
        # except Exception as e: print(e)
    return current_folder['id'], folder_tree, folder_dict

def list_folders(folder_dict, current_folder, folder_numbers):
    for x in folder_numbers:
        current_folder = current_folder['folders'][x]
    print('\nNykyinen kansio: \033[32m' + folder_dict[current_folder['id']][0] + '\033[39m')
    print('\n\033[92m{:<5}{:<35}{:<14}{}\033[39m'.format('#', 'Kansio', 'Tallenteita', 'Tallenteita ml. alikansiot'))
    for i, x in enumerate(current_folder['folders']):
        print('{:<5}{:<35}{:<14}{}'.format(str(i), x['name'], x['recordingsCount'], x['totalRecordingsCount']))
    if current_folder['recordingsCount'] == 1:
        print('Kansiossa 1 tallenne.')
    else:
        print('Kansiossa ' + str(current_folder['recordingsCount']) + ' tallennetta.')
    print()
    return current_folder

def create_folder_dict(folder_tree):
    folder_dict = {}
    folder_dict = folder_loop(folder_dict, folder_tree['folders'], folder_tree['id'], '', tuple())
    folder_dict[0] = ['(Tallennekansio)', None, '(Tallennekansio)']
    if config['Download settings'].getboolean('folder structure'):
        with open (os.path.join(os.environ['APPDATA'], 'viihdecli', 'kansiot.json'), 'w') as f:
            json.dump(folder_dict, f)
    return folder_dict

def folder_loop(folder_dict, folder, folder_id, folder_path, parent_folders):
    for x in folder:
        parent_folder_list = list(parent_folders)
        parent_folder_list.append(folder_id)
        folder_dict[x['id']] = [folder_path + x['name'], parent_folder_list, x['name']]
        if len(x['folders']) > 0:
            folder_loop(folder_dict, x['folders'], x['id'], folder_dict[x['id']][0] + ' / ', tuple(folder_dict[x['id']][1]))
    return folder_dict

def handle_recordings(folders, recording_list, headers, list_recordings = False, list_only = False, folder = None,
                     hl_set = set(), prompt = 'Käsittele tallenteita: ', trash = False):
    recordings_moved = False
    folder_dict = create_folder_dict(folders)
    sorting_key = config['General settings']['sorting key']
    sorting_dir = config['General settings']['sorting dir']
    imdb_sort = False
    all_recordings = set()
    for x in recording_list:
        all_recordings.add(x['programId'])
    all_filtered = all_recordings
    all_filtered_list = update_all_filtered_list(recording_list, all_recordings)
    print_descriptions = False
    if list_recordings:
        print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
    if list_only:
        return
    filtered_recordings = {}
    set_operations = ''
    global platform
    if config['General settings'].getboolean('print help at start') and not hl_set:
        viihdehelp.print_help('recordings', trash)
        print()
    while True:
        try:
            f_string = input(prompt).strip()
            if f_string == '':
                return recordings_moved
            if f_string.startswith(command_strings.USE_FILTER_SHORTCUT):
                shortcut = f_string[1:].strip()
                lf = len(filtered_recordings)
                dup_filters = []
                if shortcut in config['Filter shortcuts']:
                    f_list = [config['Filter shortcuts'][shortcut]]
                elif f'{shortcut}_0' in config['Filter shortcuts']:
                    i = 0
                    f_list = []
                    while f'{shortcut}_{i}' in config['Filter shortcuts']:
                        f_list.append(config['Filter shortcuts'][f'{shortcut}_{i}'])
                        i += 1
                for i, x in enumerate(f_list):
                    f_string = x
                    if f_string[0] == '!':
                        not_in = True
                        f_string = f_string[1:]
                    else:
                        not_in = False
                    f_split = f_string.split(' ', 1)
                    valid_filter, filtered_recordings = filter_recordings(recording_list, filtered_recordings, not_in, f_split[0], f_split[1].upper())
                    if valid_filter == 2:
                        dup_filters.append(i)
                if not dup_filters and f'{shortcut}_so_' in config['Filter shortcuts']:
                    so = config['Filter shortcuts'][f'{shortcut}_so_']
                    if lf == 0:
                        set_operations = so
                    else:
                        so = re.sub(r'\d+', lambda x: str(int(x.group(0)) + lf), so)
                        if not set_operations and lf > 0:
                            set_operations = '0'
                            for i in range(1, lf):
                                set_operations += f' & {i}'
                        set_operations = f'({set_operations}) & ({so})'
                elif set_operations:
                    so = str(lf)
                    # print(f'Dups: {len(dup_filters)}')
                    dups = 0
                    for i in range(lf + 1, lf + len(f_list)):
                        if i - lf - 1 not in dup_filters:
                            so += f' & {i - dups}'
                        else:
                            dups += 1
                    set_operations = f'({set_operations}) & ({so})'
                if set_operations:
                    # print(set_operations)
                    all_filtered = custom_filter(all_recordings, filtered_recordings, set_operations)
                else:
                    all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
                continue
            if f_string[0] == '!':
                not_in = True
                f_string = f_string[1:]
            else:
                not_in = False

            if f_string == command_strings.SHOW_HELP:
                print()
                viihdehelp.print_help('recordings', trash)
                print()

            elif f_string == command_strings.SHOW_FILTER_HELP:
                print()
                viihdehelp.print_help('filters')
                print()

            elif f_string == command_strings.SHOW_SORTING_HELP:
                print()
                viihdehelp.print_help('sorting', trash)
                print()

            elif f_string == command_strings.SHOW_WIDTH:
                print()
                show_terminal_width(columns, len(all_filtered_list), trash)
                print()

            elif f_string == command_strings.SHOW_QUOTA:
                account_quota = viihdeapi.get_quota(headers, platform)
                seconds = account_quota['secondsLeft']
                hours_seconds = divmod(seconds, 3600)
                minutes = hours_seconds[1] // 60
                print(f'Tallennustilaa jäljellä {hours_seconds[0]} tuntia, {minutes} minuuttia')

            elif f_string == command_strings.CLEAR_FILTERS:
                filtered_recordings = {}
                set_operations = ''
                print('Suotimet tyhjennetty.')
                all_filtered = all_recordings
                all_filtered_list = update_all_filtered_list(recording_list, all_filtered)

            elif f_string == command_strings.CLEAR_LAST_FILTER:
                print('Suodin poistettu: ' + str(filtered_recordings.popitem()[0]))
                set_operations = ''
                all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                if len(filtered_recordings) > 0:
                    print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string.startswith(command_strings.CLEAR_FILTER):
                f_split = int(f_string.split(' ', 1)[1])
                remove_filter = list(filtered_recordings)[f_split]
                filtered_recordings.pop(remove_filter)
                set_operations = ''
                print('Suodin poistettu: ' + str(remove_filter))
                all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                if len(filtered_recordings) > 0:
                    print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string == command_strings.LIST_FILTERED:
                if len(filtered_recordings) == 0:
                    all_filtered = all_recordings
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string == command_strings.LIST_FILTERS:
                print('\nAktiiviset suotimet:')
                for i, x in enumerate(list(filtered_recordings)):
                    if set_operations and str(i) not in set_operations:
                        print(f'\033[31m{str(i):<5}{x}\033[39m')
                    else:
                        print(f'{str(i):<5}{x}')
                if set_operations:
                    print('Sääntö: ' + set_operations)
                print()

            elif f_string == command_strings.REFRESH_RECORDINGS:
                if not hl_set:
                    folders = viihdeapi.get_folder_tree(headers, platform)
                    folder_dict = create_folder_dict(folders)
                    if trash:
                        recording_list = viihdeapi.get_recycle(sorting_key, sorting_dir, headers, platform)['recordings']
                    elif folder is None:
                        recording_list = viihdeapi.get_all_recordings(sorting_key, sorting_dir, headers, platform)['recordings']
                    else:
                        recording_list = viihdeapi.get_recordings(str(folder), sorting_key, sorting_dir, headers, platform)['recordings']
                    if imdb_sort:
                        recording_list = sort_recordings(recording_list, 'imdbRating', False if sorting_dir == 'asc' else True)
                    all_recordings = set()
                    for x in recording_list:
                        all_recordings.add(x['programId'])
                        # if 'folderId' in x and not x['folderId'] in folder_dict:
                            # folders = viihdeapi.get_folder_tree(headers, platform)
                            # folder_dict = create_folder_dict(folders)
                            # break
                    if not filtered_recordings:
                        all_filtered = all_recordings
                    else:
                        filtered_recordings_ = {}
                        for y in filtered_recordings.values():
                            if not y[2]:
                                rem_set = y[1] & all_recordings
                                if rem_set:
                                    dict_key = 'Piilotettu listauksesta'
                                    filtered_recordings_[dict_key] = [True, rem_set, None]
                            else:
                                filtered_recordings_ = filter_recordings(recording_list, filtered_recordings_, y[0], y[2][0], y[2][1])[1]
                        filtered_recordings = filtered_recordings_
                        if set_operations:
                            all_filtered = custom_filter(all_recordings, filtered_recordings, set_operations)
                        else:
                            all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    if filtered_recordings or config['General settings'].getboolean('print all'):
                        print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
                else:
                    print('Tallenteita ei voi päivittää duplikaatteja käsitellessä.')

            elif f_string.startswith(command_strings.REMOVE_DUPLICATES):
                if not hl_set:
                    stime = time.time()
                    f_split = f_string.split(' ', 1)
                    if len(f_split) > 1:
                        if f_split[1] in ['1', '2', '3', '4', '5', '6', '7']:
                            mode = int(f_split[1])
                            duplicate_set, duplicate_list = list_duplicates(all_filtered_list.copy(), mode)
                        else:
                            mode = 0
                            use_description = 'd' in f_split[1]
                            use_name = 'n' in f_split[1]
                            use_metadata = 'm' in f_split[1]
                            dot_index = f_split[1].find('.') + 1
                            max_dur_diff = None
                            if not dot_index == -1:
                                digits = ''
                                for i in f_split[1][dot_index:]:
                                    if i.isdigit():
                                        digits += i
                                    else:
                                        break
                                if digits: max_dur_diff = int(digits)
                            duplicate_set, duplicate_list = list_duplicates(all_filtered_list.copy(), mode, use_description, use_name, use_metadata, max_dur_diff)
                    else:
                        duplicate_set, duplicate_list = list_duplicates(all_filtered_list.copy())

                    print(f'Duplikaattien hakuun kului {time.time() - stime} sekuntia.')
                    # print(duplicate_set)
                    if duplicate_set:
                        handle_recordings(folders, duplicate_list, headers, True, True, None, duplicate_set)
                        del_prompt = input('Haluatko poistaa kaikki duplikaatit (k/e)? ')
                        if del_prompt in ['k', 'y']:
                            del_list = []
                            for x in duplicate_list:
                                if x['programId'] in duplicate_set:
                                    del_list.append(x)
                            del_list_str = ','.join(map(str,[x['programId'] for x in del_list]))
                            if config['General settings'].getboolean('recycle duplicates'):
                                if viihdeapi.delete_recordings(del_list_str, headers, platform) == 200:
                                    recordings_moved = True
                                    print('Duplikaatit siirrettiin Viihteen roskakoriin.')
                                    not_in_folder = [x['programId'] for x in del_list]
                                    recording_list = [x for x in recording_list if not x['programId'] in not_in_folder]
                                    for x in not_in_folder:
                                        all_recordings.remove(x)
                                    all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                                    folders = viihdeapi.get_folder_tree(headers, platform)
                            else:
                                if not config.has_option('Folder shortcuts','trash'):
                                    print('Valitse kansio, jota käytetään roskakorina.')
                                    trash_id, folders, folder_dict = handle_folders(folders, headers, True, folder_dict)
                                    set_folder_shortcut(trash_id, 'trash', print_msg = False)
                                else:
                                    trash_id = config['Folder shortcuts']['trash']
                                if viihdeapi.move_recordings(del_list_str, trash_id, headers, platform) == 200:
                                    recordings_moved = True
                                    print('Duplikaatit siirrettiin roskakorikansioon.')
                                    if folder is None:
                                        for x in del_list:
                                            x['folderId'] = int(trash_id)
                                    elif folder != trash_id:
                                        not_in_folder = [x['programId'] for x in del_list]
                                        recording_list = [x for x in recording_list if not x['programId'] in not_in_folder]
                                        for x in not_in_folder:
                                            all_recordings.remove(x)
                                        all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                                        all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                                    folders = viihdeapi.get_folder_tree(headers, platform)
                        else:
                            handle_recordings(folders, duplicate_list, headers, False, False, None, duplicate_set, 'Käsittele duplikaatteja: ')
                    else:
                        print('Duplikaatteja ei löydetty.')
                else:
                    print('Duplikaatteja ollaan jo käsittelemässä.')

            elif f_string.startswith(command_strings.CHANNEL_DAY):
                f_split = f_string.split(' ', 1)
                recording_info = all_filtered_list[int(f_split[1])]
                start_date = date.fromisoformat(recording_info['startTime'].split()[0])
                days_between = [start_date - timedelta(days=1), start_date + timedelta(days=2)]
                channel = recording_info['channelName']
                filter_list = [[False, 'c', channel], [True, 'b', days_between[0].isoformat()], [False, 'b', days_between[1].isoformat()]]
                filtered_recordings_ = {}
                for x in filter_list:
                    filtered_recordings_ = filter_recordings(recording_list, filtered_recordings_, x[0], x[1], x[2].upper())[1]
                all_filtered_ = update_all_filtered(all_recordings, filtered_recordings_)
                all_filtered_list_ = update_all_filtered_list(recording_list, all_filtered_)
                handle_recordings(folders, all_filtered_list_, headers, True, False, None, {recording_info['programId']}, 'Käsittele tallenteita: ')

            elif f_string.startswith(command_strings.HIDE_RECORDINGS):
                to_remove = create_number_list(f_string.split(' ', 1)[1], len(all_filtered) - 1)
                dict_key = 'Piilotettu listauksesta'
                if dict_key not in filtered_recordings:
                    filtered_recordings[dict_key] = [True, set(), None]
                for x in to_remove:
                    filtered_recordings[dict_key][1].add(all_filtered_list[x]['programId'])
                all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string.startswith(command_strings.PLAY_RECORDING):
                if trash:
                    print('Viihteen roskakorissa olevia tallenteita ei voida toistaa.')
                    print('Jos haluat katsoa tallenteen, palauta se roskakorista.')
                else:
                    f_split = f_string.split(' ', 1)
                    play_recording(str(all_filtered_list[int(f_split[1])]['programId']), headers)
                    print('Toistetaan tallenne ' + all_filtered_list[int(f_split[1])]['name'] + '.')

            elif f_string.startswith(command_strings.SHOW_DESCRIPTION):
                f_split = f_string.split(' ', 1)
                print()
                print(all_filtered_list[int(f_split[1])]['name'])
                print(all_filtered_list[int(f_split[1])].get('description'))

            elif f_string == command_strings.PRINT_DESCRIPTIONS:
                print_descriptions = not print_descriptions
                print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string.startswith(command_strings.SHOW_INFO):
                f_split = f_string.split(' ', 1)
                recording_info = all_filtered_list[int(f_split[1])]
                print()
                for x in recording_info:
                    print(x,':',recording_info[x])
                print()

            elif f_string.startswith(command_strings.SHOW_URL):
                f_split = f_string.split(' ', 1)
                recording_url = play_recording(str(all_filtered_list[int(f_split[1])]['programId']), headers, True)
                print('Tallenteen ' + all_filtered_list[int(f_split[1])]['name'] + ' URL: ' + recording_url)

            elif f_string.startswith(command_strings.DOWNLOAD_RECORDINGS):
                if trash:
                    print('Viihteen roskakorissa olevia tallenteita ei voida ladata.')
                    print('Jos haluat ladata tallenteita, palauta ne roskakorista.')
                else:
                    f_split = f_string.split(' ', 1)
                    cmd = 'viihdedl '
                    if f_split[1] == 'a':
                        if len(all_filtered) > 0:
                            print('Ladataan suodatetut tallenteet.')
                            cmd += ' '.join([str(x['programId']) for x in all_filtered_list]) + ' '
                    else:
                        dl_list = create_number_list(f_split[1], len(all_filtered) - 1)
                        for x in dl_list:
                            cmd += str(all_filtered_list[x]['programId']) + ' '
                    cmd += str(headers)
                    # print(cmd)
                    Popen(cmd, creationflags=CREATE_NEW_CONSOLE)

            elif f_string.startswith(command_strings.RESTORE_RECORDINGS):
                f_split = f_string.split(' ', 1)
                if f_split[1] == 'a':
                    if len(all_filtered) > 0:
                        res_list = all_filtered_list
                        res_list_str = ','.join(map(str,all_filtered))
                else:
                    res_list = []
                    for x in create_number_list(f_split[1], len(all_filtered) - 1):
                        res_list.append(all_filtered_list[x])
                    res_list_str = ','.join(map(str,[x['programId'] for x in res_list]))
                if viihdeapi.restore_recordings(res_list_str, headers, platform) == 200:
                    recordings_moved = True
                    print('Valitut tallenteet palautettiin Viihteen roskakorista.')
                    # for x in res_list:
                        # x['folderId'] = int(trash_id)
                    # folders = folder_tree = viihdeapi.get_folder_tree(headers, platform)

            elif f_string.startswith(command_strings.RECYCLE_RECORDINGS):
                f_split = f_string.split(' ', 1)
                if f_split[1] == 'a':
                    if len(all_filtered) > 0:
                        del_list = all_filtered_list
                        del_list_str = ','.join(map(str,all_filtered))
                else:
                    del_list = []
                    for x in create_number_list(f_split[1], len(all_filtered) - 1):
                        del_list.append(all_filtered_list[x])
                    del_list_str = ','.join(map(str,[x['programId'] for x in del_list]))
                if int(config['General settings']['move prompt min']) and len(del_list) >= int(config['General settings']['move prompt min']):
                    del_confirmation = input('Haluatko varmasti siirtää ' + str(len(del_list)) + ' tallennetta Viihteen roskakoriin (k/e)? ')
                    if del_confirmation not in ('k', 'y'):
                        print('Tallenteita ei siirretty Viihteen roskakoriin.')
                        continue
                if viihdeapi.delete_recordings(del_list_str, headers, platform) == 200:
                    recordings_moved = True
                    print('Valitut tallenteet siirrettiin Viihteen roskakoriin.')
                    not_in_folder = [x['programId'] for x in del_list]
                    recording_list = [x for x in recording_list if not x['programId'] in not_in_folder]
                    for x in not_in_folder:
                        all_recordings.remove(x)
                    all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    folders = viihdeapi.get_folder_tree(headers, platform)

            elif f_string.startswith(command_strings.DELETE_RECORDINGS):
                f_split = f_string.split(' ', 1)
                if f_split[1] == 'a':
                    if len(all_filtered) > 0:
                        del_list = all_filtered_list
                        del_list_str = ','.join(map(str,all_filtered))
                elif f_split[1] == 'd':
                    del_list = []
                    for x in all_filtered_list:
                        if x['programId'] in hl_set:
                            del_list.append(x)
                    del_list_str = ','.join(map(str,[x['programId'] for x in del_list]))
                else:
                    del_list = []
                    for x in create_number_list(f_split[1], len(all_filtered) - 1):
                        del_list.append(all_filtered_list[x])
                    del_list_str = ','.join(map(str,[x['programId'] for x in del_list]))
                if not config.has_option('Folder shortcuts','trash'):
                    print('\nValitse kansio, jota käytetään roskakorina.')
                    trash_id, folders, folder_dict = handle_folders(folders, headers, True, folder_dict)
                    set_folder_shortcut(trash_id, 'trash', print_msg = False)
                else:
                    trash_id = config['Folder shortcuts']['trash']
                if int(config['General settings']['move prompt min']) and len(del_list) >= int(config['General settings']['move prompt min']):
                    del_confirmation = input('Haluatko varmasti siirtää ' + str(len(del_list)) + ' tallennetta roskakoriin (k/e)? ')
                    if del_confirmation not in ('k', 'y'):
                        print('Tallenteita ei siirretty roskakorikansioon.')
                        continue
                if viihdeapi.move_recordings(del_list_str, trash_id, headers, platform) == 200:
                    recordings_moved = True
                    print('Valitut tallenteet siirrettiin roskakorikansioon.')
                    if folder is None:
                        for x in del_list:
                            x['folderId'] = int(trash_id)
                    elif folder != trash_id:
                        not_in_folder = [x['programId'] for x in del_list]
                        recording_list = [x for x in recording_list if not x['programId'] in not_in_folder]
                        for x in not_in_folder:
                            all_recordings.remove(x)
                        all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                        all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    folders = viihdeapi.get_folder_tree(headers, platform)

            elif f_string.startswith(command_strings.MOVE_RECORDINGS):
                f_string = f_string.split(' ', 1)[1]
                target_split = f_string.split('_', 1)
                shortcut_split = f_string.split('>', 1)
                if len(target_split) == 2:
                    folder_id = all_filtered_list[int(target_split[1].strip())]['folderId']
                    f_string = target_split[0].strip()
                elif len(shortcut_split) == 2:
                    folder_id = int(config['Folder shortcuts'][shortcut_split[1].strip()])
                    f_string = shortcut_split[0].strip()
                else:
                    print('Valitse kansio, johon tallenteet siirretään.')
                    folder_id, folders, folder_dict = handle_folders(folders, headers, True, folder_dict)
                if f_string == 'a':
                    mov_list = all_filtered_list
                else:
                    mov_list = []
                    for x in create_number_list(f_string, len(all_filtered) - 1):
                        mov_list.append(all_filtered_list[x])
                mov_list_str = ','.join(map(str,[x['programId'] for x in mov_list]))
                if int(config['General settings']['move prompt min']) and len(mov_list) >= int(config['General settings']['move prompt min']):
                    mov_confirmation = input('Haluatko varmasti siirtää ' + str(len(mov_list)) + ' tallennetta kansioon '
                                            + folder_dict[folder_id][0] + ' (k/e)? ')
                    if mov_confirmation not in ('k', 'y'):
                        print('Tallenteita ei siirretty.')
                        continue
                if viihdeapi.move_recordings(mov_list_str, folder_id, headers, platform) == 200:
                    recordings_moved = True
                    print('Valitut tallenteet siirrettiin kansioon ' + folder_dict[folder_id][0] + '.')
                    if folder is None:
                        for x in mov_list:
                            x['folderId'] = int(folder_id)
                    elif folder != folder_id:
                        not_in_folder = [x['programId'] for x in mov_list]
                        recording_list = [x for x in recording_list if not x['programId'] in not_in_folder]
                        for x in not_in_folder:
                            all_recordings.remove(x)
                        all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                        all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    folders = viihdeapi.get_folder_tree(headers, platform)

            elif f_string.startswith(command_strings.EXCLUDE_FOLDER):
                if f_string == command_strings.EXCLUDE_FOLDER:
                    if not_in:
                        print('Valitse kansio, jossa tallenteiden tulee sijaita.')
                    else:
                        print('Valitse kansio, jossa sijaitsevat tallenteet suodatetaan pois.')
                    folder_id, folders, folder_dict = handle_folders(folders, headers, True, folder_dict)
                else:
                    shortcut_split = f_string.split('>', 1)
                    f_split = f_string.split(' ', 1)[1].strip()
                    if len(shortcut_split) == 2:
                        folder_id = int(config['Folder shortcuts'][shortcut_split[1].strip()])
                    elif f_split.lstrip('-').isnumeric:
                        folder_id = all_filtered_list[int(f_split.strip())]['folderId']
                valid_filter, filtered_recordings = filter_recordings(recording_list, filtered_recordings, not_in, 'folder', [folder_id, folder_dict[folder_id][0]])
                if valid_filter == 1:
                    if set_operations:
                        set_operations += f' & {len(filtered_recordings) - 1}'
                        all_filtered = custom_filter(all_recordings, filtered_recordings, set_operations)
                    else:
                        all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
                elif valid_filter == 2:
                    print('Suodin on jo käytössä.')
                else:
                    print('Virheellinen komento.')

            elif f_string.startswith(command_strings.SORT_RECORDINGS) or f_string.startswith(command_strings.SORT_RECORDINGS_DESC):
                f_split = f_string.split(' ', 1)
                sort_orders = {'s': 'startTimeUTC', 'e': 'endTimeUTC', 'n': 'name', 'd': 'duration', 'c': 'channelName', 'i': 'imdbRating'}
                if trash: sort_orders['r'] = 'removalDateUTC'
                if f_split[1] in sort_orders:
                    sorting_dir, rev_order = ('asc', False) if f_string.startswith(command_strings.SORT_RECORDINGS) else ('desc', True)
                    imdb_sort = False
                    if f_split[1] in ['s', 'e', 'r']:
                        sorting_key = sort_orders[f_split[1]][:-3]
                    elif f_split[1] == 'i':
                        imdb_sort = True
                    else:
                        sorting_key = sort_orders[f_split[1]]
                    recording_list = sort_recordings(recording_list, sort_orders[f_split[1]], rev_order)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    if config['General settings'].getboolean('print all') or len(filtered_recordings) > 0:
                        print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)

            elif f_string == command_strings.RELOAD_CONFIG:
                config['Folder shortcuts'] = {}
                config['Download folders'] = {}
                config.read(config_path)
                columns.read(columns_path)
                platform = config['Download settings']['platform']
                print('Asetukset päivitetty.')

            elif f_string == command_strings.OPEN_CONFIG:
                print('Avataan ' + config_path + ' Muistiossa.')
                print('Muokattuasi asetustiedostoa kirjoita ' + command_strings.RELOAD_CONFIG + ' päivittääksesi asetukset.')
                webbrowser.open(config_path)

            elif f_string == command_strings.OPEN_COLUMNS:
                print('Avataan ' + columns_path + ' Muistiossa.')
                print('Muokattuasi asetustiedostoa kirjoita ' + command_strings.RELOAD_CONFIG + ' päivittääksesi asetukset.')
                webbrowser.open(columns_path)

            # elif f_string.startswith(command_strings.SET_FOLDER_SHORTCUT_NUMBER):
                # f_split = f_string.split(' ', 1)[1].split(':', 1)
                # print(f_split)
                # folder_id = all_filtered_list[int(f_split[1].strip())]['folderId']
                # set_folder_shortcut(folder_id, f_split[0].strip())
                # print('Luotiin pikavalinta ' + f_split[0].strip() + ' kansiolle ' + str(folder_dict[int(folder_id)][0]) + '.')

            elif f_string.startswith(command_strings.SET_FILTER_SHORTCUT):
                f_split = f_string.split(' ', 1)[1].split('|', 1)
                f_split = [x.strip() for x in f_split]
                f_list = [f'{"!" if x[0] else ""}{x[2][0]} {x[2][1]}'.lower() for x in list(filtered_recordings.values())]
                s_o = ''
                if len(f_split) == 1:
                    f_list = [f_list[-1]]
                elif f_split[1] == 'a':
                    s_o = set_operations
                else:
                    f_list_ = []
                    for x in create_number_list(f_split[1], len(f_list) - 1):
                        f_list_.append(f_list[x])
                    f_list = f_list_
                set_filter_shortcut(f_split[0], f_list, s_o)

            elif f_string == command_strings.LIST_FILTER_SHORTCUTS:
                list_filter_shortcuts()

            elif f_string.startswith(command_strings.SET_FOLDER_SHORTCUT):
                f_split = f_string.split(' ', 1)
                shortcut_split = f_split[1].split('|', 1)
                if len(shortcut_split) == 2:
                    f_split[1] = shortcut_split[0].strip()
                    folder_id = all_filtered_list[int(shortcut_split[1].strip())]['folderId']
                else:
                    folder_id, folders, folder_dict = handle_folders(folders, headers, True, folder_dict)
                set_folder_shortcut(folder_id, f_split[1], folder_dict[int(folder_id)][0])

            elif f_string.startswith(command_strings.DL_FOLDER_SHORTCUT):
                set_dl_folder(config['Download folders'][f_string.split(' ', 1)[1]])

            elif f_string.startswith(command_strings.SET_DL_FOLDER):
                set_dl_folder(f_string.split(' ', 1)[1])

            elif f_string.startswith(command_strings.SET_DL_FOLDER_SHORTCUT):
                f_split = f_string.split(' ', 1)[1].split('|', 1)
                if len(f_split) == 1:
                    f_split.append(config['Download settings']['download folder'])
                set_dl_folder_shortcut(f_split[1].strip(), f_split[0].strip())

            elif f_string == command_strings.LIST_DL_FOLDERS:
                list_dl_folders()

            elif f_string == command_strings.LIST_FOLDER_SHORTCUTS:
                list_folder_shortcuts(folder_dict)

            elif f_string == 'd_filtered':
                print(filtered_recordings)

            elif f_string.startswith(command_strings.CUSTOM_FILTER):
                allowed_char = '0123456789a()|&-^ '
                f_o = f_string.split(' ', 1)[1].strip()
                # f_o = re.sub(' +', ' ', f_o)
                if all(c in allowed_char for c in f_o):
                    all_filtered = custom_filter(all_recordings, filtered_recordings, f_o)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
                    set_operations = f_o
                else:
                    print('Komento saa sisältää alun cf:n lisäksi ainostaan numeroita, kirjaimen a ja merkit (, ), |, &, - ja ^.')
            else:
                f_split = f_string.split(' ', 1)
                valid_filter, filtered_recordings = filter_recordings(recording_list, filtered_recordings, not_in, f_split[0], f_split[1].upper())
                if valid_filter == 1:
                    if set_operations:
                        set_operations += f' & {len(filtered_recordings) - 1}'
                        all_filtered = custom_filter(all_recordings, filtered_recordings, set_operations)
                    else:
                        all_filtered = update_all_filtered(all_recordings, filtered_recordings)
                    all_filtered_list = update_all_filtered_list(recording_list, all_filtered)
                    print_recordings(columns, folder_dict, all_filtered_list, hl_set, print_descriptions, trash)
                elif valid_filter == 2:
                    print('Suodin on jo käytössä.')
                else:
                    print('Virheellinen komento.')
        except:
            print('Virheellinen komento.')
            # raise
        # except Exception as e: print(e)

def sort_recordings(recording_list, sort_by, reverse_order):
    if sort_by == 'imdbRating':
        imdb_list = []
        no_imdb_list = []
        for x in recording_list:
            if 'imdbRating' in x:
                imdb_list.append(x)
            else:
                no_imdb_list.append(x)
        imdb_list = sorted(imdb_list, key = lambda recording: recording['name'], reverse = False)
        no_imdb_list = sorted(no_imdb_list, key = lambda recording: recording['name'], reverse = False)
        imdb_list = sorted(imdb_list, key = lambda recording: recording[sort_by], reverse = reverse_order)
        new_recording_list = no_imdb_list + imdb_list
    else:
        new_recording_list = sorted(recording_list, key = lambda recording: recording[sort_by], reverse = reverse_order)
    return new_recording_list

def update_all_filtered(all_recordings, filtered_recordings):
    y = 0
    all_filtered = set()
    for x in list(filtered_recordings):
        if y == 0:
            if filtered_recordings[x][0] is False:
                all_filtered = filtered_recordings[x][1]
            else:
                all_filtered = all_recordings - filtered_recordings[x][1]
        elif filtered_recordings[x][0] is True:
            all_filtered = all_filtered - filtered_recordings[x][1]
        else:
            all_filtered = all_filtered & filtered_recordings[x][1]
        y += 1
    all_filtered = all_filtered & all_recordings
    return all_filtered

def custom_filter(all_recordings, filtered_recordings, operations):
    # fv = [x[1] for x in list(filtered_recordings.values())]
    fv = list(filtered_recordings.values())
    # print(fv)
    all_filtered = set()
    operations = re.sub(r'\d+', r'fv[\g<0>]', operations)
    for i, x in enumerate(fv):
        # operations = operations.replace(str(i), f'fv[{i}]')
        if x[0]:
            operations = operations.replace(f'fv[{i}]', f'(a - fv[{i}])')
        # print(operations)
    operations = operations.replace('a', 'all_recordings')
    operations = operations.replace(']', '][1]')
    # print(operations)
    # print(len(fv))
    all_filtered = eval(operations)
    return all_filtered

def update_all_filtered_list(recording_list, recording_set):
    all_filtered_list = [None] * len(recording_set)
    i = 0
    for x in recording_list:
        if x['programId'] in recording_set:
            if not 'folderId' in x:
                x['folderId'] = 0
            all_filtered_list[i] = x
            i += 1
    return all_filtered_list

def handle_input(headers):
    download = True
    while download:
        recording = input('Tallenteen tai kansion osoite: ').rstrip(' /')
        if recording in ['https://elisaviihde.fi/tallenteet',
                         'https://elisaviihde.fi/tallenteet/sivu/1', '0']:
            download_folder('0', headers)
        elif recording.isnumeric():
            cmd = 'viihdedl ' + recording + ' ' + str(headers)
            Popen(cmd, creationflags=CREATE_NEW_CONSOLE)
        else:
            folder_split = recording.rsplit('/', 3)
            if folder_split[0] == 'https://elisaviihde.fi/tallenteet/kansio':
                download_folder(folder_split[1], headers)
            else:
                recording_split = recording.rsplit('/', 1)
                if ( recording_split[0] == 'https://elisaviihde.fi/ohjelmaopas/ohjelma' or
                     recording_split[0] == 'https://elisaviihde.fi/tallenteet/katso' or
                     recording_split[0] == 'https://elisaviihde.fi/tallenne/katso' ):
                    cmd = 'viihdedl ' + recording_split[1] + ' ' + str(headers)
                    Popen(cmd, creationflags=CREATE_NEW_CONSOLE)
                else:
                    break



def clean_name(recording_name):
    cleaned_name = recording_name.upper()
    for x in ['ELOKUVA:', 'UUSI KINO:', 'KINO KLASSIKKO:', 'KINO SUOMI:', 'KINO:', 'SUBLEFFA:']:
        if cleaned_name.startswith(x):
            cleaned_name = cleaned_name.replace(x, '')
            break
    if cleaned_name.endswith(')'):
        cleaned_name = cleaned_name.rsplit('(', 1)[0]
    cleaned_name = cleaned_name.strip(':!. ')
    return cleaned_name

def clean_description(description):
    cleaned_description = description.upper()
    for x in ['(U)', 'UUSI SARJA ALKAA', 'UUSI KAUSI ALKAA', 'UUSI KAUSI', '(KAAPELI-TV)', 'SUOMEN-ENSI-ILTA', 'SUOMEN ENSI-ILTA', 'SUOMEN TV-ENSI-ILTA', 'TV-ENSI-ILTA', 'SUORA LÄHETYS']:
        if x in cleaned_description:
            cleaned_description = cleaned_description.replace(x, '')
            break
    if cleaned_description.endswith(')'):
        cleaned_description = cleaned_description.rsplit('(', 1)[0]
    cleaned_description = cleaned_description.strip(':!. ')
    return cleaned_description

def list_duplicates(recording_list, mode = 2, use_description = True, use_name = True, use_metadata = False, max_dur_diff = None):
    folder_length = len(recording_list)
    # duplicates = [[None] * 10 for i in range(folder_length)]
    # stime = time.time()
    if mode != 0:
        use_name = False if mode == 5 else True
        use_description = False if mode == 5 else True
        use_metadata = False if mode == 2 else True
    if use_name:
        names_cleaned = [clean_name(x['name']) for x in recording_list]
    if use_description:
        descriptions_cleaned = [clean_description(x['description']) if 'description' in x else '' for x in recording_list]
    if use_metadata:
        metadata_ids = [x['metadataId'] if 'metadataId' in x else 0 for x in recording_list]
    if max_dur_diff is not None:
        use_dur_diff = True
        durations = [x['duration'] for x in recording_list]
        max_dur_diff_sec = 60 * max_dur_diff
    else:
        use_dur_diff = False
    duplicates_found = [False] * folder_length
    duplicate_list = [None] * folder_length
    duplicate_set = set()
    i = 0

    if mode == 1:
        # Name AND description AND metadataId
        print('Haetaan duplikaatteja (nimi JA kuvaus JA metadataId).')
        for x in range(folder_length):
            if duplicates_found[x] or not metadata_ids[x] or not descriptions_cleaned[x]:
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or not metadata_ids[y] or not descriptions_cleaned[y]:
                    continue
                if ( metadata_ids[x] == metadata_ids[y] and names_cleaned[x] == names_cleaned[y] and
                     (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x])) ):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1

    elif mode == 2:
        # Name AND description
        print('Haetaan duplikaatteja (nimi JA kuvaus).')
        for x in range(folder_length):
            if duplicates_found[x] or not descriptions_cleaned[x]:
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or not descriptions_cleaned[y]:
                    continue
                if names_cleaned[x] == names_cleaned[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x])):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1

    elif mode == 3:
        # Description AND (name OR metadataId)
        print('Haetaan duplikaatteja (kuvaus JA (nimi TAI metadataId)).')
        for x in range(folder_length):
            if duplicates_found[x] or not descriptions_cleaned[x]:
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or not descriptions_cleaned[y]:
                    continue
                if ( ((metadata_ids[x] and  metadata_ids[x] == metadata_ids[y]) or names_cleaned[x] == names_cleaned[y]) and
                     (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x])) ):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1
    elif mode == 4:
        # MetadataId AND (name OR description)
        print('Haetaan duplikaatteja (metadataId JA (nimi tai kuvaus)).')
        for x in range(folder_length):
            if duplicates_found[x] or not metadata_ids[x]:
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or not metadata_ids[y]:
                    continue
                if ( metadata_ids[x] == metadata_ids[y] and (names_cleaned[x] == names_cleaned[y] or
                     (descriptions_cleaned[y] and descriptions_cleaned[x].startswith(descriptions_cleaned[y])) or
                     (descriptions_cleaned[x] and descriptions_cleaned[y].startswith(descriptions_cleaned[x]))) ):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1
    elif mode == 5:
        # MetadataId
        print('Haetaan duplikaatteja (metadataId).')
        for x in range(folder_length):
            if duplicates_found[x] or not metadata_ids[x]:
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or not metadata_ids[y]:
                    continue
                if metadata_ids[x] == metadata_ids[y]:
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1
    elif mode == 6:
        # Name AND (description OR metadataId)
        print('Haetaan duplikaatteja (nimi JA (kuvaus TAI metadataId)).')
        for x in range(folder_length):
            if duplicates_found[x] or (not metadata_ids[x] and not descriptions_cleaned[x]):
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or (not metadata_ids[y] and not descriptions_cleaned[y]):
                    continue
                if ( names_cleaned[x] == names_cleaned[y] and ((metadata_ids[x] and  metadata_ids[x] == metadata_ids[y]) or
                     (descriptions_cleaned[y] and descriptions_cleaned[x].startswith(descriptions_cleaned[y])) or
                     (descriptions_cleaned[x] and descriptions_cleaned[y].startswith(descriptions_cleaned[x]))) ):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1
    elif mode == 7:
        # (Name AND description) OR metadataId
        print('Haetaan duplikaatteja ((nimi JA kuvaus) TAI metadataId).')
        for x in range(folder_length):
            if duplicates_found[x] or (not metadata_ids[x] and not descriptions_cleaned[x]):
                continue
            for y in range(x + 1, folder_length):
                if duplicates_found[y] or (not metadata_ids[y] and not descriptions_cleaned[y]):
                    continue
                if ( (metadata_ids[x] and metadata_ids[x] == metadata_ids[y]) or (names_cleaned[x] == names_cleaned[y] and
                     ((descriptions_cleaned[y] and descriptions_cleaned[x].startswith(descriptions_cleaned[y])) or
                     (descriptions_cleaned[x] and descriptions_cleaned[y].startswith(descriptions_cleaned[x])))) ):
                    if not duplicates_found[x]:
                        duplicates_found[x] = True
                        duplicate_list[i] = recording_list[x]
                        i += 1
                    duplicates_found[y] = True
                    duplicate_list[i] = recording_list[y]
                    duplicate_set.add(recording_list[y]['programId'])
                    i += 1

    else:
        if use_name:
            if use_description:
                if use_metadata:
                    if use_dur_diff:
                        # Name, description, metadataId, durations
                        print('Haetaan duplikaatteja (nimi, kuvaus, metadataId, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y] or not descriptions_cleaned[y]:
                                    continue
                                if (metadata_ids[x] == metadata_ids[y] and names_cleaned[x] == names_cleaned[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))
                                    and abs(durations[x] - durations[y]) <= max_dur_diff_sec):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Name, description, metadataId
                        print('Haetaan duplikaatteja (nimi, kuvaus, metadataId).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y] or not descriptions_cleaned[y]:
                                    continue
                                if (metadata_ids[x] == metadata_ids[y] and names_cleaned[x] == names_cleaned[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1

                else:
                    if use_dur_diff:
                        # Name, description, durations
                        print('Haetaan duplikaatteja (nimi, kuvaus, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not descriptions_cleaned[y]:
                                    continue
                                if (names_cleaned[x] == names_cleaned[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))
                                    and abs(durations[x] - durations[y]) <= max_dur_diff_sec):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Name, description
                        print('Haetaan duplikaatteja (nimi, kuvaus).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not descriptions_cleaned[y]:
                                    continue
                                if names_cleaned[x] == names_cleaned[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x])):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
            else:
                if use_metadata:
                    if use_dur_diff:
                        # Name, metadata, durations
                        print('Haetaan duplikaatteja (nimi, metadataId, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y]:
                                    continue
                                if names_cleaned[x] == names_cleaned[y] and metadata_ids[x] == metadata_ids[y] and abs(durations[x] - durations[y]) <= max_dur_diff_sec:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Name, metadata
                        print('Haetaan duplikaatteja (nimi, metadataId).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y]:
                                    continue
                                if names_cleaned[x] == names_cleaned[y] and metadata_ids[x] == metadata_ids[y]:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                else:
                    if use_dur_diff:
                        # Name, durations
                        print('Haetaan duplikaatteja (nimi, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y]:
                                    continue
                                if names_cleaned[x] == names_cleaned[y] and abs(durations[x] - durations[y]) <= max_dur_diff_sec:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Name
                        print('Haetaan duplikaatteja (nimi).')
                        for x in range(folder_length):
                            if duplicates_found[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y]:
                                    continue
                                if names_cleaned[x] == names_cleaned[y]:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
        else:
            if use_description:
                if use_metadata:
                    if use_dur_diff:
                        # Description, metadataId, durations
                        print('Haetaan duplikaatteja (kuvaus, metadataId, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y] or not descriptions_cleaned[y]:
                                    continue
                                if (metadata_ids[x] == metadata_ids[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))
                                    and abs(durations[x] - durations[y]) <= max_dur_diff_sec):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Description, metadataId
                        print('Haetaan duplikaatteja (kuvaus, metadataId).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y] or not descriptions_cleaned[y]:
                                    continue
                                if (metadata_ids[x] == metadata_ids[y] and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1

                else:
                    if use_dur_diff:
                        # Description, durations
                        print('Haetaan duplikaatteja (kuvaus, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not descriptions_cleaned[y]:
                                    continue
                                if (abs(durations[x] - durations[y]) <= max_dur_diff_sec and (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x]))):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Description
                        print('Haetaan duplikaatteja (kuvaus).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not descriptions_cleaned[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not descriptions_cleaned[y]:
                                    continue
                                if (descriptions_cleaned[x].startswith(descriptions_cleaned[y]) or descriptions_cleaned[y].startswith(descriptions_cleaned[x])):
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
            else:
                if use_metadata:
                    if use_dur_diff:
                        # Metadata, durations
                        print('Haetaan duplikaatteja (metadataId, ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y]:
                                    continue
                                if metadata_ids[x] == metadata_ids[y] and abs(durations[x] - durations[y]) <= max_dur_diff_sec:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                    else:
                        # Metadata
                        print('Haetaan duplikaatteja (metadataId).')
                        for x in range(folder_length):
                            if duplicates_found[x] or not metadata_ids[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y] or not metadata_ids[y]:
                                    continue
                                if metadata_ids[x] == metadata_ids[y]:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1
                else:
                    if use_dur_diff:
                        # Durations
                        print('Haetaan duplikaatteja (ero kestossa korkeintaan ' + str(max_dur_diff) + ' minuuttia).')
                        for x in range(folder_length):
                            if duplicates_found[x]:
                                continue
                            for y in range(x + 1, folder_length):
                                if duplicates_found[y]:
                                    continue
                                if abs(durations[x] - durations[y]) <= max_dur_diff_sec:
                                    if not duplicates_found[x]:
                                        duplicates_found[x] = True
                                        duplicate_list[i] = recording_list[x]
                                        i += 1
                                    duplicates_found[y] = True
                                    duplicate_list[i] = recording_list[y]
                                    duplicate_set.add(recording_list[y]['programId'])
                                    i += 1

    return duplicate_set, duplicate_list[:i]



from .command_strings import *

general_help = [['ENTER', '', 'Hyväksy/Palaa/Poistu'],
               [SHOW_HELP, '', 'Näytä tämä listaus'],
               [SHOW_QUOTA, '', 'Näytä jäljellä oleva tallennustila'],
               [LIST_FOLDER_SHORTCUTS, '', 'Listaa kansioiden pikavalinnat'],
               [LIST_DL_FOLDERS, '', 'Näydä nykyinen latauskansio ja pikavalinnat'],
               [SET_DL_FOLDER, 'POLKU', 'Aseta latauskansioksi POLKU'],
               [DL_FOLDER_SHORTCUT, 'NIMI', 'Aseta pikavalinta NIMI latauskansioksi'],
               [SET_DL_FOLDER_SHORTCUT, 'NIMI', 'Lisää nykyinen latauskansio pikavalinnaksi NIMI'],
               [SET_DL_FOLDER_SHORTCUT, 'NIMI | POLKU', 'Lisää latauskansio POLKU pikavalinnaksi NIMI'],
               [RELOAD_CONFIG, '', 'Päivitä asetukset tiedostoista settings.ini ja columns.ini'],
               [OPEN_CONFIG, '', 'Avaa settings.ini Muistiossa'],
               [OPEN_COLUMNS, '', 'Avaa columns.ini Muistiossa']]

folder_help = [general_help[0], general_help[1], general_help[2],
              ['', '#', 'Siirry kansioon # (numero)'],
              [ROOT_FOLDER, '', 'Siirry pääkansioon'],
              [FOLDER_UP, '', 'Siirry yläkansioon'],
              [LIST_FOLDERS, '', 'Listaa kansiot'],
              [NEW_FOLDER, 'NIMI', 'Luo uusi kansio NIMI'],
              [DELETE_FOLDER, '', 'Poista nykyinen kansio'],
              [DELETE_FOLDER, ' #', 'Poista kansio #'],
              [LIST_RECORDINGS, '', 'Listaa nykyisen kansion tallenteet'],
              [REFRESH_RECORDINGS, '', 'Päivitä kansiot'],
              [DOWNLOAD_FOLDER, '', 'Lataa nykyisen kansion tallenteet'], general_help[2],
              [SET_SHORTCUT, 'NIMI', 'Lisää nykyinen kansio pikavalinnaksi NIMI']] + general_help[3:]

recording_help = [general_help[0], general_help[1], general_help[2],
                 [RESTORE_RECORDINGS, '##', 'Palauta tallenteet ## Elisa Viihteen roskakorista'],
                 [LIST_FILTERED, '', 'Listaa suodatetut tallenteet'],
                 [SHOW_WIDTH, '', 'Näytä tallenteiden listaukseen tarvittava terminaalin leveys'],
                 [PRINT_DESCRIPTIONS, '', 'Näytä/piilota tallenteiden kuvaukset'],
                 [SHOW_SORTING_HELP, '', 'Näytä lajitteluavaimet'],
                 [SORT_RECORDINGS, '%', 'Järjestä tallenteet nousevasti avaimen % mukaisesti (ks. LUEMINUT)'],
                 [SORT_RECORDINGS_DESC, '%', 'Järjestä tallenteet laskevasti avaimen % mukaisesti (ks. LUEMINUT)'],
                 [REFRESH_RECORDINGS, '', 'Päivitä tallenteet ja kansiot'],
                 [LIST_FILTERS, '', 'Listaa käytössä olevat suotimet'],
                 [SHOW_FILTER_HELP, '', 'Listaa mahdolliset suotimet'],
                 [CLEAR_FILTERS, '', 'Poista kaikki suotimet'],
                 [CLEAR_LAST_FILTER, '', 'Poista viimeisin suodin'],
                 [CLEAR_FILTER, 'X', 'Poista suodin numero X'],
                 [CUSTOM_FILTER, ' OPERAATIOT', 'Joukko-operaatiot suodatetuille joukoille (ks. LUEMINUT)'],
                 [SET_FILTER_SHORTCUT, 'NIMI', 'Lisää viimeisin suodin pikavalinnaksi NIMI'],
                 [SET_FILTER_SHORTCUT, 'NIMI | X', 'Lisää suodin numero X pikavalinnaksi NIMI'],
                 [SET_FILTER_SHORTCUT, 'NIMI | KOMENTO', 'Lisää KOMENTO pikavalinnaksi NIMI'],
                 [USE_FILTER_SHORTCUT, 'NIMI', 'Käytä pikavalinnan NIMI suodinta'],
                 [LIST_FILTER_SHORTCUTS, '', 'Listaa suodinten pikavalinnat'],
                 [CHANNEL_DAY, '#', 'Näytä tallennetta # edeltävät ja seuraavat tallenteet'],
                 [HIDE_RECORDINGS, '##', 'Suodata pois tallenteet ##'],
                 [EXCLUDE_FOLDER, '', 'Suodata pois valittavassa kansiossa sijaitsevat tallenteet'],
                 [EXCLUDE_FOLDER, ' #', 'Suodata pois tallenteen # kansiossa sijaitsevat tallenteet'],
                 [EXCLUDE_FOLDER, ' > NIMI', 'Suodata pois kansiossa, jonka pikavalinta on NIMI, sijaitsevat tallenteet'],
                 [REMOVE_DUPLICATES, '', 'Etsi duplikaatit (sama nimi ja kuvaus)'],
                 [REMOVE_DUPLICATES, ' X', 'Etsi duplikaatit, hakumoodi X (ks. LUEMINUT)'],
                 [PLAY_RECORDING, '#', 'Toista tallenne # (numero)'],
                 [SHOW_DESCRIPTION, '#', 'Näytä tallenteen # kuvaus'],
                 [SHOW_INFO, '#', 'Näytä tallenteen # tiedot'],
                 [SHOW_URL, '#', 'Näytä tallenteen # URL'],
                 [DOWNLOAD_RECORDINGS, '##', 'Lataa tallenteet ##'],
                 [DELETE_RECORDINGS, '##', 'Siirrä tallenteet ## roskakorikansioon'],
                 [RECYCLE_RECORDINGS, '##', 'Poista tallenteet ## Elisa Viihteen roskakoriin'],
                 [MOVE_RECORDINGS, '##', 'Siirrä tallenteet ## valittavaan kansioon'],
                 [MOVE_RECORDINGS, '## > NIMI', 'Siirrä tallenteet ## kansioon, jonka pikavalinta on NIMI'],
                 [MOVE_RECORDINGS, '## _ #', 'Siirrä tallenteet ## kansioon, jossa tallenne # sijaitsee'], general_help[2],
                 [SET_FOLDER_SHORTCUT, 'NIMI', 'Lisää valittava kansio pikavalinnaksi NIMI'],
                 [SET_FOLDER_SHORTCUT, 'NIMI | #', 'Lisää tallenteen # sisältävä kansio pikavalinnaksi NIMI']] + general_help[3:]

filter_help = [[F_NAME, 'TEKSTI', 'Tallenteen nimi sisältää tekstin'],
              [F_DESCRIPTION, 'TEKSTI', 'Tallenteen kuvaus sisältää tekstin'],
              [F_CHANNEL, 'TEKSTI', 'Tallenteen kanavanimi sisältää tekstin'],
              [F_DURATION, 'X tai HH:MM', 'Tallenteen pituus vähintään X min tai HH tuntia, MM minuuttia'],
              [F_START_TIME, 'YYYY-MM-DD tai DD.MM.YYYY', 'Tallennus alkoi ennen päivämäärää'],
              [F_END_TIME, 'YYYY-MM-DD tai DD.MM.YYYY', 'Tallennus päättyi ennen päivämäärää'],
              [F_TIME_REMAINING, 'X', 'Tallenteen katseluaikaa jäljellä alle X vuorokautta'],
              [F_WEEKDAY, 'VIIKONPÄIVÄN LYHENNE', 'Tallennus alkoi viikonpäivänä'],
              [F_TIME, 'HH:MM', 'Tallennus alkoi ennen kellonaikaa'],
              [F_ACTOR, 'SUKUNIMI, ETUNIMI', 'Näyttelijä esintyy tallenteella'],
              [F_DIRECTOR, 'SUKUNIMI, ETUNIMI', 'Tallenne on ohjaajan ohjaama'],
              [F_IMDB, 'X.Y', 'Tallenteen IMDB-pisteet vähintään X.Y'],
              [F_SHOWTYPE, 'TEKSTI', 'Tallenteen showtype (metadatassa) sisältää tekstin'],
              [F_GENRE, 'TEKSTI', 'Jokin tallenteen genreistä (metadatassa) sisältää tekstin'],
              [F_SERIES, 'TEKSTI', 'Sarjan nimi sisältää tekstin'],
              [F_EPISODE_NAME, 'TEKSTI', 'Sarjan jakson nimi sisältää tekstin'],
              [F_SEASON, 'XX', 'Sarjan kausi XX'],
              [F_EPISODE, 'XX', 'Sarjan jakso XX']]

sorting_help = [['s', 'alkamisaika'],
               ['e', 'päättymisaika'],
               ['n', 'nimi'],
               ['d', 'kesto'],
               ['c', 'kanava'],
               ['i', 'IMDB-arvosana'],
               ['r', 'poistuu lopullisesti']]

def print_help(position, trash = False):
    if position == 'folders':
        print('\033[92m{:<20}{}\033[39m'.format('Komento', 'Selite'))
        for x in folder_help:
            help_line = '{:<20}{}'.format(x[0] + x[1], x[2])
            print(help_line)
    elif position == 'recordings':
        print('\033[92m{:<20}{}\033[39m'.format('Komento', 'Selite'))
        if trash:
            for x in recording_help:
                if not x[0] in [PLAY_RECORDING, SHOW_URL, DOWNLOAD_RECORDINGS, RECYCLE_RECORDINGS]:
                    help_line = '{:<20}{}'.format(x[0] + x[1], x[2])
                    print(help_line)
        else:
            for x in recording_help:
                if not x[0] in [RESTORE_RECORDINGS]:
                    help_line = '{:<20}{}'.format(x[0] + x[1], x[2])
                    print(help_line)
    elif position == 'filters':
        print('\033[92m{:<30}{}\033[39m'.format('Komento', 'Selite'))
        for x in filter_help:
            help_line = '{:<30}{}'.format(x[0] + ' ' + x[1], x[2])
            print(help_line)
    elif position == 'sorting':
        l = len(sorting_help)
        print('\033[92m{:<12}{}\033[39m'.format('Avain', 'Selite'))
        for i, x in enumerate(sorting_help):
            if not trash and i == l - 1:
                break
            help_line = '{:<12}{}'.format(x[0], x[1])
            print(help_line)



# Folder view
ROOT_FOLDER = '/'
FOLDER_UP = '..'
LIST_FOLDERS = '.'
NEW_FOLDER = 'n '
LIST_RECORDINGS = 'l'
DOWNLOAD_FOLDER = 'dl'
SET_SHORTCUT = 'fs '                # command shortcutName
DELETE_FOLDER = 'del'

# Recordings view
CLEAR_FILTERS = '.'
CLEAR_LAST_FILTER = '..'
CLEAR_FILTER = '. '
LIST_FILTERED = 'l'
LIST_FILTERS = 'f'
CUSTOM_FILTER = 'cf'
SET_FILTER_SHORTCUT = '+ '
USE_FILTER_SHORTCUT = ', '
LIST_FILTER_SHORTCUTS = 'ff'
REFRESH_RECORDINGS = 'r'
REMOVE_DUPLICATES = 'rd'            # n for checking names only
CHANNEL_DAY = 'cd '
HIDE_RECORDINGS = 'rem '
PLAY_RECORDING = 'p '
SHOW_DESCRIPTION = 'd '
PRINT_DESCRIPTIONS = 'pd'
SHOW_INFO = 'i '
SHOW_URL = 'u '
DOWNLOAD_RECORDINGS = 'dl '         # a for all
DELETE_RECORDINGS = 'del '          # a for all, d for duplicates
MOVE_RECORDINGS = 'm '              # command recordings _ recordingNumber OR command recordings > shortcutName OR command recordings, a for all
EXCLUDE_FOLDER = 'ex'               # command recordingNumber OR command > shortcutName OR command
SORT_RECORDINGS = 's '              # s for starttime, e for endtime, n for name, d for duration, c for channel, i for IMDB rating
SORT_RECORDINGS_DESC = 'sd '
SET_FOLDER_SHORTCUT = 'fs '         # command shortcutName
# SET_FOLDER_SHORTCUT_NUMBER = 'fn '  # command shortcutName : recordingNumber
RECYCLE_RECORDINGS = 'rm '
RESTORE_RECORDINGS = 're '
SHOW_FILTER_HELP = 'fh'
SHOW_SORTING_HELP = 'sh'
SHOW_WIDTH = 'sw'

# Both views
SHOW_HELP = 'h'
SHOW_QUOTA = 'q'
DL_FOLDER_SHORTCUT = 'ds '          # command dlShortcutName
SET_DL_FOLDER_SHORTCUT = 'dp '      # command dlShortcutName : dlFolderPath
SET_DL_FOLDER = 'df '               # command dlFolderPath
LIST_DL_FOLDERS = 'ld'
LIST_FOLDER_SHORTCUTS = 'lf'
RELOAD_CONFIG = 'rc'
OPEN_CONFIG = 'o'
OPEN_COLUMNS = 'oc'

# Filters
F_ACTOR = 'a'                       # lastName, firstName (wildcard * at end possible)
F_DIRECTOR = 'di'                   # lastName, firstName (wildcard * at end possible)
F_NAME = 'n'
F_DESCRIPTION = 'de'
F_CHANNEL = 'c'                     # channelName1 | channelName2 | channelName3 ...
F_SHOWTYPE = 'st'
F_GENRE = 'g'
F_SERIES = 'ser'
F_EPISODE_NAME = 'en'
F_DURATION = 'dur'                  # minutes or hh:mm
F_START_TIME = 'b'                  # start date before
F_END_TIME = 'be'                   # start date before
F_TIME_REMAINING = 'tr'
F_WEEKDAY = 'w'                     # weekday1 | weekday2 ...
F_TIME = 't'                        # start time before
F_IMDB = 'im'                       # includes (st movie)
F_SEASON = 'se'                     # season
F_EPISODE = 'ep'                    # episode



import json
import requests

BASE_URL = 'https://api-viihde-gateway.dc1.elisa.fi/'

def login(client_secret, api_key, username, password):
    # Get access code.
    payload = {'client_id': 'external', 'client_secret': client_secret,
               'response_type': 'code', 'scopes': []}
    headers_ = {'content-type': 'application/json', 'apikey': api_key}
    access_code_json = requests.post(BASE_URL + 'auth/authorize/access-code',
                                  json=payload, headers=headers_)
    access_code = json.loads(access_code_json.text)['code']

    # Get access token.
    payload = {'grant_type': 'authorization_code', 'username': username,
               'password': password, 'client_id': 'external', "code": access_code}
    headers_ = {'content-type': 'application/x-www-form-urlencoded', 'apikey': api_key}
    access_token_json = requests.post(BASE_URL + 'auth/authorize/access-token',
                                   data=payload, headers=headers_)
    access_token = json.loads(access_token_json.text)['access_token']
    headers = {'Authorization': 'Bearer '+ access_token,  'apikey': api_key}
    return headers

def get_folder_tree(headers, platform):
    folders = requests.get(BASE_URL + 'rest/npvr/folders?v=2'
                           + '&platform=' + platform + '&appVersion=1.0', headers=headers)
    return json.loads(folders.text)

def get_folders(folder_id, headers, platform):
    folders = requests.get(BASE_URL + 'rest/npvr/folders/' + folder_id
                           + '?platform=' + platform + '&appVersion=1.0', headers=headers)
    # time.sleep(0.05)
    # print(folders.status_code)
    return json.loads(folders.text)

def get_recordings(folder_id, sort_by, sort_order, headers, platform):
    recordings = requests.get(BASE_URL + 'rest/npvr/recordings/folder/' + folder_id +
                              '?v=2.1&page=0&sortBy=' + sort_by + '&sortOrder=' + sort_order
                              + '&pageSize=99999&platform=' + platform + '&appVersion=1.0'
                              '&includeMetadata=true', headers=headers)
    return json.loads(recordings.text)

def get_recycle(sort_by, sort_order, headers, platform):
    recordings = requests.get(BASE_URL + 'rest/npvr/recordings/recycle'
                                  '?v=1&page=0&sortBy=' + sort_by + '&sortOrder=' + sort_order
                                  + '&pageSize=99999&platform=' + platform + '&appVersion=1.0'
                                  '&includeMetadata=true', headers=headers)
    return json.loads(recordings.text)

def delete_recordings(recording_list, headers, platform):
    payload = {'programId': recording_list}
    del_request = requests.put(BASE_URL + 'rest/npvr/recordings/delete?v=2&platform=' + platform
                               + '&appVersion=1.0', data=payload, headers=headers)
    return del_request.status_code

def restore_recordings(recording_list, headers, platform):
    payload = {'programId': recording_list}
    restore_request = requests.put(BASE_URL + 'rest/npvr/recordings/recycle?v=1&platform=' + platform
                               + '&appVersion=1.0', data=payload, headers=headers)
    return restore_request.status_code

def get_all_recordings(sort_by, sort_order, headers, platform):
    recordings = requests.get(BASE_URL + 'rest/npvr/recordings'
                                  '?v=2.1&page=0&sortBy=' + sort_by + '&sortOrder=' + sort_order
                                  + '&pageSize=99999&platform=' + platform + '&appVersion=1.0'
                                  '&includeMetadata=true', headers=headers)
    return json.loads(recordings.text)

def get_recording_info(recording_id, headers, platform):
    recording_info = requests.get(BASE_URL + 'rest/npvr/recordings/info/' + recording_id
                                 + '?v=2.1&platform=online&appVersion=1.0&includeMetadata=true',
                                 headers=headers)
    return json.loads(recording_info.text)

def get_recording_url(recording_id, headers, platform):
    payload = '{"cdnServiceOptions":["s_ttml"],"protocol":"hls","applicationVersion":"1","deviceId":"123","drmPlatform":"ios"}'
    headers_ = {'content-type': 'application/json; charset=UTF-8', 'Authorization': headers['Authorization']}
    recording_url = requests.post('https://watchable-api.dc.elisa.fi/V3/recordings/play-options/'
                                + recording_id + '/' + platform, data=payload, headers=headers_)
    # return json.loads(recording_url.text)['url']
    temp_url = json.loads(recording_url.text)['requestRouterUrl']
    perm_url = requests.get(temp_url).url
    return perm_url

def get_recording_url_(recording_id, headers, platform):
    recording_url = requests.get('https://rest-api.elisaviihde.fi/rest/npvr/recordings/url/'
                                + recording_id + '?v=2&platform=' + platform
                                + '&appVersion=1.0', headers=headers)
    # return json.loads(recording_url.text)['url']
    temp_url = json.loads(recording_url.text)['url']
    perm_url = requests.get(temp_url).url
    return perm_url

def move_recordings(recording_list, folder_id, headers, platform):
    payload = {'programId': recording_list, 'folderId': folder_id}
    move_request = requests.put(BASE_URL + 'rest/npvr/recordings/move?v=2&platform=' + platform
                               + '&appVersion=1.0', data=payload, headers=headers)
    return move_request.status_code

def create_folder(name, parent_folder_id, headers, platform):
    if parent_folder_id == '0':
        payload = {'name': name}
    else:
        payload = {'name': name, 'parentFolderId': parent_folder_id}
    create_request = requests.put(BASE_URL + 'rest/npvr/folders?v=0.1&platform=' + platform
                                 + '&appVersion=1.0', data=payload, headers=headers)
    return json.loads(create_request.text)['folderId']

def delete_folder(folder_id, headers, platform):
    payload = {'folderId': folder_id}
    del_request = requests.delete(BASE_URL + 'rest/npvr/folders/' + folder_id + '?v=2&platform='
                                   + platform + '&appVersion=1.0', data=payload, headers=headers)
    return del_request.status_code

def get_quota(headers, platform):
    quota_info = requests.get(BASE_URL + 'rest/npvr/recordings/quota?v=2&platform='
                              + platform, headers=headers)
    return json.loads(quota_info.text)

def get_account_info(headers):
    account_info = requests.get(BASE_URL + 'rest/myaccount/devices,services,netpvr,authentication?v=2',
                                 headers=headers)
    return json.loads(account_info.text)

# def mark_as_watched(recording_id, headers, platform):
    # a=1

# def set_play_position(recording_id, play_position, headers, platform):
    # a=1

# def search_recording(search_string, headers, platform):
    # a=1



from datetime import datetime, time
import locale
from . import command_strings
locale.setlocale(locale.LC_ALL, '')

def filter_recordings(recording_list, filtered_recordings, not_in, f, arguments):
    # filtered_recordings = filteredRecordings_
    # print(recording_list)
    f_len = len(filtered_recordings)
    dict_key = ''
    if not_in:
        dict_key = 'EI: '
    # stime = time.time()
    if f == 'folder':
        if not_in:
            dict_key = 'Kansiossa: ' + arguments[1]
        else:
            dict_key = 'Ei kansiossa: ' + arguments[1]
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if arguments[0] != x['folderId']:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_ACTOR:
        # stime = time.time()
        names_split = arguments.split('|')
        name_list = []
        for x in names_split:
            name_split = x.split(',')
            name_split = [y.strip() for y in name_split]
            last_name_only = len(name_split) == 1
            first_name_only = name_split[0] == ''
            last_name_starts = not first_name_only and name_split[0].endswith('*')
            first_name_starts = not last_name_only and name_split[0].endswith('*')
            name_split = [y.rstrip('*') for y in name_split]
            name_list.append({'names': name_split, 'last_name_only': last_name_only, 'first_name_only': first_name_only,
                             'last_name_starts': last_name_starts, 'first_name_starts': first_name_starts})
        dict_key += 'Näyttelijä: ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'actors' in x:
                    for y in x['actors']:
                        for z in name_list:
                            if z['last_name_only']:
                                if z['last_name_starts']:
                                    if y['lastName'].upper().startswith(z['names'][0]):
                                        filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if y['lastName'].upper() == z['names'][0]:
                                        filtered_recordings[dict_key][1].add(x['programId'])
                            elif z['first_name_only']:
                                if z['first_name_starts']:
                                    if y['firstName'].upper().startswith(z['names'][1]):
                                        filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if y['firstName'].upper() == z['names'][1]:
                                        filtered_recordings[dict_key][1].add(x['programId'])
                            else:
                                if z['last_name_starts']:
                                    if z['first_name_starts']:
                                        if y['lastName'].upper().startswith(z['names'][0]) and y['firstName'].upper().startswith(z['names'][1]):
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                    else:
                                        if y['lastName'].upper().startswith(z['names'][0]) and y['firstName'].upper() == z['names'][1]:
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if z['first_name_starts']:
                                        if y['lastName'].upper() == z['names'][0] and y['firstName'].upper().startswith(z['names'][1]):
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                    else:
                                        if y['lastName'].upper() == z['names'][0] and y['firstName'].upper() == z['names'][1]:
                                            filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings
        # print(time.time()-stime)

    elif f == command_strings.F_DIRECTOR:
        names_split = arguments.split('|')
        name_list = []
        for x in names_split:
            name_split = x.split(',')
            name_split = [y.strip() for y in name_split]
            last_name_only = len(name_split) == 1
            first_name_only = name_split[0] == ''
            last_name_starts = not first_name_only and name_split[0].endswith('*')
            first_name_starts = not last_name_only and name_split[0].endswith('*')
            name_split = [y.rstrip('*') for y in name_split]
            name_list.append({'names': name_split, 'last_name_only': last_name_only, 'first_name_only': first_name_only,
                             'last_name_starts': last_name_starts, 'first_name_starts': first_name_starts})
        dict_key += 'Ohjaaja: ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'directors' in x:
                    for y in x['directors']:
                        for z in name_list:
                            if z['last_name_only']:
                                if z['last_name_starts']:
                                    if y['lastName'].upper().startswith(z['names'][0]):
                                        filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if y['lastName'].upper() == z['names'][0]:
                                        filtered_recordings[dict_key][1].add(x['programId'])
                            elif z['first_name_only']:
                                if z['first_name_starts']:
                                    if y['firstName'].upper().startswith(z['names'][1]):
                                        filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if y['firstName'].upper() == z['names'][1]:
                                        filtered_recordings[dict_key][1].add(x['programId'])
                            else:
                                if z['last_name_starts']:
                                    if z['first_name_starts']:
                                        if y['lastName'].upper().startswith(z['names'][0]) and y['firstName'].upper().startswith(z['names'][1]):
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                    else:
                                        if y['lastName'].upper().startswith(z['names'][0]) and y['firstName'].upper() == z['names'][1]:
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                else:
                                    if z['first_name_starts']:
                                        if y['lastName'].upper() == z['names'][0] and y['firstName'].upper().startswith(z['names'][1]):
                                            filtered_recordings[dict_key][1].add(x['programId'])
                                    else:
                                        if y['lastName'].upper() == z['names'][0] and y['firstName'].upper() == z['names'][1]:
                                            filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_NAME:
        if not_in:
            dict_key = 'Nimi ei sisällä: ' + arguments
        else:
            dict_key = 'Nimi sisältää: ' + arguments
        if not dict_key in filtered_recordings.keys():
            in_name = arguments.split('|')
            in_name = [x.strip() for x in in_name]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                for y in in_name:
                    if y in x['name'].upper():
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_DESCRIPTION:
        if not_in:
            dict_key = 'Kuvaus ei sisällä: ' + arguments
        else:
            dict_key = 'Kuvaus sisältää: ' + arguments
        if not dict_key in filtered_recordings.keys():
            in_description = arguments.split('|')
            in_description = [x.strip() for x in in_description]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                for y in in_description:
                    if 'description' in x and y in x['description'].upper():
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_CHANNEL:
        dict_key += 'Kanavat: ' + arguments
        if not dict_key in filtered_recordings.keys():
            channels = arguments.split('|')
            channels = [x.strip() for x in channels]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                for y in channels:
                    if y in x['channelName'].upper():
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_SHOWTYPE:
        dict_key += 'Showtype: ' + arguments
        if not dict_key in filtered_recordings.keys():
            showtypes = arguments.split('|')
            showtypes = [x.strip() for x in showtypes]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                for y in showtypes:
                    if 'showType' in x and y in x['showType'].upper():
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_GENRE:
        dict_key += 'Genre: ' + arguments
        if not dict_key in filtered_recordings.keys():
            genres = arguments.split('|')
            genres = [x.strip() for x in genres]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'genres' in x:
                    for y in x['genres']:
                        for z in genres:
                            if z in y['name'].upper():
                                filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_SERIES:
        dict_key += 'Sarja: ' + arguments
        if not dict_key in filtered_recordings.keys():
            series = arguments.split('|')
            series = [x.strip() for x in series]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'series' in x and 'title' in x['series']:
                    for y in series:
                        if y in x['series']['title'].upper():
                            filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_EPISODE_NAME:
        dict_key += 'Episodin nimi: ' + arguments
        if not dict_key in filtered_recordings.keys():
            series = arguments.split('|')
            series = [x.strip() for x in series]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'series' in x and 'episodeName' in x['series']:
                    for y in series:
                        if y in x['series']['episodeName'].upper():
                            filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_SEASON:
        dict_key += 'Kausi: ' + arguments
        if not dict_key in filtered_recordings.keys():
            seasons = arguments.replace('|', ' ').split()
            seasons = [int(x.strip()) for x in seasons]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'series' in x and 'season' in x['series'] and int(x['series']['season']) in seasons:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_EPISODE:
        dict_key += 'Jakso: ' + arguments
        if not dict_key in filtered_recordings.keys():
            episodes = arguments.replace('|', ' ').split()
            episodes = [int(x.strip()) for x in episodes]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'series' in x and 'episode' in x['series'] and int(x['series']['episode']) in episodes:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_DURATION:
        if not_in:
            dict_key = 'Kesto alle ' + arguments + ' minuuttia'
        else:
            dict_key = 'Kesto vähintään ' + arguments + ' minuuttia'
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            argument_split = arguments.split(':')
            if len(argument_split) == 1:
                duration = 60 * int(argument_split[0])
            else:
                duration = 3600 * int(argument_split[0]) + 60 * int(argument_split[1])
            for x in recording_list:
                if x['duration'] >= duration:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_START_TIME:
        if not_in:
            dict_key = 'Tallennus alkoi aikaisintaan ' + arguments
        else:
            dict_key = 'Tallennus alkoi ennen ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            if '.' in arguments:
                d = datetime.strptime(arguments, '%x')
            else:
                d = datetime.fromisoformat(arguments)
            d_utc = d.timestamp()*1000
            for x in recording_list:
                if x['startTimeUTC'] < d_utc:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_END_TIME:
        if not_in:
            dict_key = 'Tallennus päättyi aikaisintaan ' + arguments
        else:
            dict_key = 'Tallennus päättyi ennen ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            if '.' in arguments:
                d = datetime.strptime(arguments, '%x')
            else:
                d = datetime.fromisoformat(arguments)
            d_utc = d.timestamp()*1000
            for x in recording_list:
                if x['endTimeUTC'] < d_utc:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_WEEKDAY:
        dict_key += 'Viikonpäivät: ' + arguments
        if not dict_key in filtered_recordings.keys():
            weekdays = arguments.split('|')
            weekdays = [x.strip() for x in weekdays]
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                for y in weekdays:
                    if datetime.fromisoformat(x['startTime']).strftime('%a').upper() == y:
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_TIME:
        if not_in:
            dict_key = 'Tallennusaika jälkeen ' + arguments
        else:
            dict_key = 'Tallennusaika ennen ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            hours_minutes = [int(x.strip()) for x in arguments.split(':', 1)]
            t = time(hour = hours_minutes[0], minute = hours_minutes[1])
            for x in recording_list:
                if datetime.fromisoformat(x['startTime']).time() < t:
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == 'tr':
        if not_in:
            dict_key = 'Katseluaikaa jäljellä vähintään ' + arguments + ' vuorokautta'
        else:
            dict_key = 'Katseluaikaa jäljellä alle ' + arguments + ' vuorokautta'
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if int(x['remainingTime']['years']) * 365 + int(x['remainingTime']['months']) * 30 + int(x['remainingTime']['days']) < int(arguments):
                    filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings

    elif f == command_strings.F_IMDB:
        if not_in:
            dict_key = 'IMDB-pisteet alle ' + arguments
        else:
            dict_key = 'IMDB-pisteet vähintään ' + arguments
        if not dict_key in filtered_recordings.keys():
            filtered_recordings[dict_key] = [not_in, set(), [f, arguments]]
            for x in recording_list:
                if 'imdbRating' in x:
                    # if x['imdbRating'] >= float(arguments) and x['showType'] == 'MOVIE':
                    if x['imdbRating'] >= float(arguments):
                        filtered_recordings[dict_key][1].add(x['programId'])
        else:
            return 2, filtered_recordings
    valid_filter = 1 if len(filtered_recordings) > f_len else 0
    return valid_filter, filtered_recordings
    # print(filtered_recordings)



from setuptools import setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

with open('src/version.py') as fp:
    exec(fp.read())

setup(
    name = 'viihdecli',
    description = 'Command line interface for handling Elisa Viihde recordings',
    long_description = long_description,
    long_description_content_type='text/markdown',
    url = 'https://github.com/Qotscha/viihdecli',
    author = 'Qotscha',
    version = __version__,
    packages = ['viihdecli'],
    package_dir = {'viihdecli': 'src'},
    python_requires='>=3.8',
    install_requires = [
        'keyring',
        'requests',
        'viihdexdl'
        ],
    entry_points = {
        'console_scripts': [
            'viihdecli = viihdecli.__main__:main',
            'viihdedl = viihdecli.viihdedl:main'
            ]
        },
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Win32 (MS Windows)',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
        ],
    project_urls = {'Documentation': 'https://github.com/Qotscha/viihdecli/blob/main/LUEMINUT.md'}
    )



import configparser
import os.path
import re

config_path = os.path.join(os.environ['APPDATA'], 'viihdecli', 'settings.ini')
config = configparser.ConfigParser()
config.read(config_path)
dl_settings = config['Download settings']
dl_folder = dl_settings['download folder']

def create_filename(recording_info):
    filename = re.sub(r'[\\/*?:"<>|]',"_",recording_info['name'])

    # Add series and episode information to the filename if available (e.g. S02E14)
    if dl_settings.getboolean('season episode filename'):
        if ('series' in recording_info and 'season' in recording_info['series']
            and 'episode' in recording_info['series']):
            filename = filename.strip()
            if filename.endswith(')'):
                filename = filename.rsplit('(', 1)[0].strip()
            season = recording_info['series']['season']
            episode = recording_info['series']['episode']
            filename += ' S' + str(season).zfill(2) + 'E' + str(episode).zfill(2)
            return filename

    recording_time = recording_info['startTime'].split()
    filename += ' ' + (recording_time[0].replace('-', '.') + ' '
                + recording_time[1].rsplit(':',1)[0].replace(':', '.'))
    return filename

def create_text_file(recording_info):
    return recording_info['description']
